@@ -1,5 +1,6 @@
 import asyncio
-from datetime import datetime, timezone
+import contextlib
+from datetime import datetime, timedelta, timezone
 import pathlib
 import sqlite3
 import sys
@@ -25,7 +26,7 @@ from zigpy.zcl.foundation import Status as ZCLStatus
 from zigpy.zdo import types as zdo_t
 
 from tests.async_mock import AsyncMock, MagicMock, patch
-from tests.conftest import App, make_ieee
+from tests.conftest import make_app, make_ieee
 from tests.test_backups import backup_factory  # noqa: F401
 
 
@@ -43,27 +44,19 @@ def auto_kill_aiosqlite():
         except ValueError:
             pass
         else:
-            try:
+            with contextlib.suppress(zigpy.appdb.sqlite3.ProgrammingError):
                 conn.close()
-            except zigpy.appdb.sqlite3.ProgrammingError:
-                pass
 
         thread._running = False
 
 
-async def make_app(database_file):
+async def make_app_with_db(database_file):
     if isinstance(database_file, pathlib.Path):
         database_file = str(database_file)
 
-    with patch("zigpy.ota.OTA.initialize", AsyncMock()):
-        app = await App.new(
-            conf.ZIGPY_SCHEMA(
-                {
-                    conf.CONF_DATABASE: database_file,
-                    conf.CONF_DEVICE: {conf.CONF_DEVICE_PATH: "/dev/null"},
-                }
-            )
-        )
+    app = make_app({conf.CONF_DATABASE: database_file})
+    await app._load_db()
+
     return app
 
 
@@ -114,20 +107,20 @@ def fake_get_device(device):
 async def test_no_database(tmp_path):
     with patch("zigpy.appdb.PersistingListener.new", AsyncMock()) as db_mock:
         db_mock.return_value.load.side_effect = AsyncMock()
-        await make_app(None)
+        await make_app_with_db(None)
     assert db_mock.return_value.load.call_count == 0
 
     db = tmp_path / "test.db"
     with patch("zigpy.appdb.PersistingListener.new", AsyncMock()) as db_mock:
         db_mock.return_value.load.side_effect = AsyncMock()
-        await make_app(db)
+        await make_app_with_db(db)
     assert db_mock.return_value.load.call_count == 1
 
 
 @patch("zigpy.device.Device.schedule_initialize", new=mock_dev_init(True))
 async def test_database(tmp_path):
     db = tmp_path / "test.db"
-    app = await make_app(db)
+    app = await make_app_with_db(db)
     ieee = make_ieee()
     relays_1 = [t.NWK(0x1234), t.NWK(0x2345)]
     relays_2 = [t.NWK(0x3456), t.NWK(0x4567)]
@@ -150,9 +143,9 @@ async def test_database(tmp_path):
     ep.profile_id = 49246
     ep.device_type = profiles.zll.DeviceType.COLOR_LIGHT
     app.device_initialized(dev)
-    clus._update_attribute(0, 99)
-    clus._update_attribute(4, bytes("Custom", "ascii"))
-    clus._update_attribute(5, bytes("Model", "ascii"))
+    clus.update_attribute(0, 99)
+    clus.update_attribute(4, bytes("Custom", "ascii"))
+    clus.update_attribute(5, bytes("Model", "ascii"))
     clus.listener_event("cluster_command", 0)
     clus.listener_event("general_command")
     dev.relays = relays_1
@@ -187,8 +180,8 @@ async def test_database(tmp_path):
     dev = app.get_device(custom_ieee)
     app.device_initialized(dev)
     dev.relays = relays_2
-    dev.endpoints[1].level._update_attribute(0x0011, 17)
-    dev.endpoints[99].level._update_attribute(0x0011, 17)
+    dev.endpoints[1].level.update_attribute(0x0011, 17)
+    dev.endpoints[99].level.update_attribute(0x0011, 17)
     assert dev.endpoints[1].in_clusters[0x0008]._attr_cache[0x0011] == 17
     assert dev.endpoints[99].in_clusters[0x0008]._attr_cache[0x0011] == 17
     custom_dev_last_seen = dev.last_seen
@@ -198,7 +191,7 @@ async def test_database(tmp_path):
 
     # Everything should've been saved - check that it re-loads
     with patch("zigpy.quirks.get_device", fake_get_device):
-        app2 = await make_app(db)
+        app2 = await make_app_with_db(db)
     dev = app2.get_device(ieee)
     assert dev.endpoints[1].device_type == profiles.zha.DeviceType.PUMP
     assert dev.endpoints[2].device_type == 0xFFFD
@@ -225,7 +218,7 @@ async def test_database(tmp_path):
     app.handle_leave(99, ieee)
     await app2.shutdown()
 
-    app3 = await make_app(db)
+    app3 = await make_app_with_db(db)
     assert ieee in app3.devices
 
     async def mockleave(*args, **kwargs):
@@ -233,12 +226,12 @@ async def test_database(tmp_path):
 
     app3.devices[ieee].zdo.leave = mockleave
     await app3.remove(ieee)
-    for i in range(1, 20):
+    for _i in range(1, 20):
         await asyncio.sleep(0)
     assert ieee not in app3.devices
     await app3.shutdown()
 
-    app4 = await make_app(db)
+    app4 = await make_app_with_db(db)
     assert ieee not in app4.devices
     dev = app4.get_device(custom_ieee)
     assert dev.relays is None
@@ -248,7 +241,7 @@ async def test_database(tmp_path):
 @patch("zigpy.device.Device.schedule_group_membership_scan", MagicMock())
 async def _test_null_padded(tmp_path, test_manufacturer=None, test_model=None):
     db = tmp_path / "test.db"
-    app = await make_app(db)
+    app = await make_app_with_db(db)
     ieee = make_ieee()
     with patch(
         "zigpy.device.Device.schedule_initialize",
@@ -265,14 +258,14 @@ async def _test_null_padded(tmp_path, test_manufacturer=None, test_model=None):
     clus = ep.add_input_cluster(0)
     ep.add_output_cluster(1)
     app.device_initialized(dev)
-    clus._update_attribute(4, test_manufacturer)
-    clus._update_attribute(5, test_model)
+    clus.update_attribute(4, test_manufacturer)
+    clus.update_attribute(5, test_model)
     clus.listener_event("cluster_command", 0)
     clus.listener_event("zdo_command")
     await app.shutdown()
 
     # Everything should've been saved - check that it re-loads
-    app2 = await make_app(db)
+    app2 = await make_app_with_db(db)
     dev = app2.get_device(ieee)
     assert dev.endpoints[3].device_type == profiles.zha.DeviceType.PUMP
     assert dev.endpoints[3].in_clusters[0]._attr_cache[4] == test_manufacturer
@@ -335,7 +328,7 @@ async def test_groups(mock_request, tmp_path):
     mock_request.return_value = [ZCLStatus.SUCCESS, group_id]
 
     db = tmp_path / "test.db"
-    app = await make_app(db)
+    app = await make_app_with_db(db)
     ieee = make_ieee()
     app.handle_join(99, ieee, 0)
 
@@ -370,7 +363,7 @@ async def test_groups(mock_request, tmp_path):
     del app, dev, dev_b, ep, ep_b
 
     # Everything should've been saved - check that it re-loads
-    app2 = await make_app(db)
+    app2 = await make_app_with_db(db)
     dev2 = app2.get_device(ieee)
     assert group_id in app2.groups
     group = app2.groups[group_id]
@@ -387,7 +380,7 @@ async def test_groups(mock_request, tmp_path):
     await app2.shutdown()
     del app2, dev2, dev2_b
 
-    app3 = await make_app(db)
+    app3 = await make_app_with_db(db)
     dev3 = app3.get_device(ieee)
     assert group_id in app3.groups
     group = app3.groups[group_id]
@@ -404,7 +397,7 @@ async def test_groups(mock_request, tmp_path):
     await app3.shutdown()
     del app3, dev3, dev3_b
 
-    app4 = await make_app(db)
+    app4 = await make_app_with_db(db)
     dev4 = app4.get_device(ieee)
     assert group_id in app4.groups
     assert not app4.groups[group_id]
@@ -413,7 +406,7 @@ async def test_groups(mock_request, tmp_path):
     await app4.shutdown()
     del app4, dev4
 
-    app5 = await make_app(db)
+    app5 = await make_app_with_db(db)
     assert not app5.groups
     await app5.shutdown()
 
@@ -423,7 +416,7 @@ async def test_attribute_update(tmp_path, dev_init):
     """Test attribute update for initialized and uninitialized devices."""
 
     db = tmp_path / "test.db"
-    app = await make_app(db)
+    app = await make_app_with_db(db)
     ieee = make_ieee()
     with patch(
         "zigpy.device.Device.schedule_initialize",
@@ -439,20 +432,26 @@ async def test_attribute_update(tmp_path, dev_init):
     ep.status = zigpy.endpoint.Status.ZDO_INIT
     ep.profile_id = 260
     ep.device_type = profiles.zha.DeviceType.PUMP
-    clus = ep.add_input_cluster(0)
-    ep.add_output_cluster(1)
-    clus._update_attribute(4, test_manufacturer)
-    clus._update_attribute(5, test_model)
+    clus = ep.add_input_cluster(0x0000)
+    ep.add_output_cluster(0x0001)
+    clus.update_attribute(0x0004, test_manufacturer)
+    clus.update_attribute(0x0005, test_model)
     app.device_initialized(dev)
     await app.shutdown()
 
+    attr_update_time = clus._attr_last_updated[0x0004]
+
     # Everything should've been saved - check that it re-loads
-    app2 = await make_app(db)
+    app2 = await make_app_with_db(db)
     dev = app2.get_device(ieee)
     assert dev.is_initialized == dev_init
     assert dev.endpoints[3].device_type == profiles.zha.DeviceType.PUMP
-    assert dev.endpoints[3].in_clusters[0]._attr_cache[4] == test_manufacturer
-    assert dev.endpoints[3].in_clusters[0]._attr_cache[5] == test_model
+
+    clus = dev.endpoints[3].in_clusters[0x0000]
+    assert clus._attr_cache[0x0004] == test_manufacturer
+    assert clus._attr_cache[0x0005] == test_model
+
+    assert (attr_update_time - clus._attr_last_updated[0x0004]) < timedelta(seconds=0.1)
 
     await app2.shutdown()
 
@@ -516,7 +515,7 @@ async def test_topology(tmp_path):
     nwk = 0x9876
 
     db = tmp_path / "test.db"
-    app = await make_app(db)
+    app = await make_app_with_db(db)
     app.handle_join(nwk, ieee, 0x0000)
 
     dev = app.get_device(ieee)
@@ -569,7 +568,7 @@ async def test_topology(tmp_path):
     del dev
 
     # Everything should've been saved - check that it re-loads
-    app2 = await make_app(db)
+    app2 = await make_app_with_db(db)
     app2.get_device(ieee)
 
     assert len(app2.topology.neighbors[ieee]) == 2
@@ -586,7 +585,7 @@ async def test_topology(tmp_path):
 @patch("zigpy.device.Device.schedule_initialize", new=mock_dev_init(True))
 async def test_device_rejoin(tmp_path):
     db = tmp_path / "test.db"
-    app = await make_app(db)
+    app = await make_app_with_db(db)
     ieee = make_ieee()
     nwk = 199
     app.handle_join(nwk, ieee, 0)
@@ -599,13 +598,13 @@ async def test_device_rejoin(tmp_path):
     clus = ep.add_input_cluster(0)
     ep.add_output_cluster(1)
     app.device_initialized(dev)
-    clus._update_attribute(4, "Custom")
-    clus._update_attribute(5, "Model")
+    clus.update_attribute(4, "Custom")
+    clus.update_attribute(5, "Model")
     await app.shutdown()
 
     # Everything should've been saved - check that it re-loads
     with patch("zigpy.quirks.get_device", fake_get_device):
-        app2 = await make_app(db)
+        app2 = await make_app_with_db(db)
     dev = app2.get_device(ieee)
     assert dev.nwk == nwk
     assert dev.endpoints[1].device_type == profiles.zha.DeviceType.PUMP
@@ -620,7 +619,7 @@ async def test_device_rejoin(tmp_path):
         app2.device_initialized(dev)
     await app2.shutdown()
 
-    app3 = await make_app(db)
+    app3 = await make_app_with_db(db)
     dev = app3.get_device(ieee)
     assert dev.nwk == nwk + 1
     assert dev.endpoints[1].device_type == profiles.zha.DeviceType.PUMP
@@ -633,7 +632,7 @@ async def test_device_rejoin(tmp_path):
 @patch("zigpy.device.Device.schedule_initialize", new=mock_dev_init(True))
 async def test_stopped_appdb_listener(tmp_path):
     db = tmp_path / "test.db"
-    app = await make_app(db)
+    app = await make_app_with_db(db)
     ieee = make_ieee()
     app.handle_join(99, ieee, 0)
 
@@ -647,14 +646,14 @@ async def test_stopped_appdb_listener(tmp_path):
     app.device_initialized(dev)
 
     with patch("zigpy.appdb.PersistingListener._save_attribute") as mock_attr_save:
-        clus._update_attribute(0, 99)
-        clus._update_attribute(4, bytes("Custom", "ascii"))
-        clus._update_attribute(5, bytes("Model", "ascii"))
+        clus.update_attribute(0, 99)
+        clus.update_attribute(4, bytes("Custom", "ascii"))
+        clus.update_attribute(5, bytes("Model", "ascii"))
         await app.shutdown()
         assert mock_attr_save.call_count == 3
 
-        clus._update_attribute(0, 100)
-        for i in range(100):
+        clus.update_attribute(0, 100)
+        for _i in range(100):
             await asyncio.sleep(0)
         assert mock_attr_save.call_count == 3
 
@@ -667,7 +666,7 @@ async def test_invalid_node_desc(tmp_path):
     nwk_1 = 0x1111
 
     db = tmp_path / "test.db"
-    app = await make_app(db)
+    app = await make_app_with_db(db)
     app.handle_join(nwk_1, ieee_1, 0)
 
     dev_1 = app.get_device(ieee_1)
@@ -681,7 +680,7 @@ async def test_invalid_node_desc(tmp_path):
     await app.shutdown()
 
     # Everything should've been saved - check that it re-loads
-    app2 = await make_app(db)
+    app2 = await make_app_with_db(db)
     dev_2 = app2.get_device(ieee=ieee_1)
     assert dev_2.node_desc is None
     assert dev_2.nwk == dev_1.nwk
@@ -723,7 +722,7 @@ async def test_unsupported_attribute(tmp_path, dev_init):
     """Test adding unsupported attributes for initialized and uninitialized devices."""
 
     db = tmp_path / "test.db"
-    app = await make_app(db)
+    app = await make_app_with_db(db)
     ieee = make_ieee()
     with patch(
         "zigpy.device.Device.schedule_initialize",
@@ -738,15 +737,15 @@ async def test_unsupported_attribute(tmp_path, dev_init):
     ep.device_type = profiles.zha.DeviceType.PUMP
     clus = ep.add_input_cluster(0)
     ep.add_output_cluster(1)
-    clus._update_attribute(4, "Custom")
-    clus._update_attribute(5, "Model")
+    clus.update_attribute(4, "Custom")
+    clus.update_attribute(5, "Model")
     app.device_initialized(dev)
     clus.add_unsupported_attribute(0x0010)
     clus.add_unsupported_attribute("physical_env")
     await app.shutdown()
 
     # Everything should've been saved - check that it re-loads
-    app2 = await make_app(db)
+    app2 = await make_app_with_db(db)
     dev = app2.get_device(ieee)
     assert dev.is_initialized == dev_init
     assert dev.endpoints[3].device_type == profiles.zha.DeviceType.PUMP
@@ -765,7 +764,7 @@ async def test_unsupported_attribute(tmp_path, dev_init):
         return [[rar0010]]
 
     # Now lets remove an unsupported attribute and make sure it is removed
-    app3 = await make_app(db)
+    app3 = await make_app_with_db(db)
     dev = app3.get_device(ieee)
     assert dev.is_initialized == dev_init
     assert dev.endpoints[3].device_type == profiles.zha.DeviceType.PUMP
@@ -775,18 +774,18 @@ async def test_unsupported_attribute(tmp_path, dev_init):
     await cluster.read_attributes([0x0010], allow_cache=False)
     assert 0x0010 not in dev.endpoints[3].in_clusters[0].unsupported_attributes
     assert "location_desc" not in dev.endpoints[3].in_clusters[0].unsupported_attributes
-    assert "Not Removed" == dev.endpoints[3].in_clusters[0].get(0x0010)
+    assert dev.endpoints[3].in_clusters[0].get(0x0010) == "Not Removed"
     assert 0x0011 in dev.endpoints[3].in_clusters[0].unsupported_attributes
     assert "physical_env" in dev.endpoints[3].in_clusters[0].unsupported_attributes
     await app3.shutdown()
 
     # Everything should've been saved - check that it re-loads
-    app4 = await make_app(db)
+    app4 = await make_app_with_db(db)
     dev = app4.get_device(ieee)
     assert dev.is_initialized == dev_init
     assert dev.endpoints[3].device_type == profiles.zha.DeviceType.PUMP
     assert 0x0010 not in dev.endpoints[3].in_clusters[0].unsupported_attributes
-    assert "Not Removed" == dev.endpoints[3].in_clusters[0].get(0x0010)
+    assert dev.endpoints[3].in_clusters[0].get(0x0010) == "Not Removed"
     assert "location_desc" not in dev.endpoints[3].in_clusters[0].unsupported_attributes
     assert 0x0011 in dev.endpoints[3].in_clusters[0].unsupported_attributes
     assert "physical_env" in dev.endpoints[3].in_clusters[0].unsupported_attributes
@@ -798,7 +797,7 @@ async def test_load_unsupp_attr_wrong_cluster(tmp_path):
     """Test loading unsupported attribute from the wrong cluster."""
 
     db = tmp_path / "test.db"
-    app = await make_app(db)
+    app = await make_app_with_db(db)
 
     ieee = make_ieee()
     app.handle_join(99, ieee, 0)
@@ -810,8 +809,8 @@ async def test_load_unsupp_attr_wrong_cluster(tmp_path):
     ep.device_type = profiles.zha.DeviceType.PUMP
     clus = ep.add_input_cluster(0)
     ep.add_output_cluster(1)
-    clus._update_attribute(4, "Custom")
-    clus._update_attribute(5, "Model")
+    clus.update_attribute(4, "Custom")
+    clus.update_attribute(5, "Model")
     app.device_initialized(dev)
     await app.shutdown()
     del clus
@@ -819,7 +818,7 @@ async def test_load_unsupp_attr_wrong_cluster(tmp_path):
     del dev
 
     # add unsupported attr for missing endpoint
-    app = await make_app(db)
+    app = await make_app_with_db(db)
     dev = app.get_device(ieee)
     ep = dev.endpoints[3]
     clus = ep.add_input_cluster(2)
@@ -830,13 +829,63 @@ async def test_load_unsupp_attr_wrong_cluster(tmp_path):
     del dev
 
     # reload
-    app = await make_app(db)
+    app = await make_app_with_db(db)
+    await app.shutdown()
+
+
+@patch.object(Device, "schedule_initialize", new=mock_dev_init(True))
+async def test_load_unsupp_attr_missing_endpoint(tmp_path):
+    """Test loading unsupported attribute from the wrong cluster."""
+
+    db = tmp_path / "test.db"
+    app = await make_app_with_db(db)
+
+    ieee = make_ieee()
+    app.handle_join(99, ieee, 0)
+
+    dev = app.get_device(ieee)
+
+    ep = dev.add_endpoint(3)
+    ep.status = zigpy.endpoint.Status.ZDO_INIT
+    ep.profile_id = 260
+    ep.device_type = profiles.zha.DeviceType.PUMP
+    clus = ep.add_input_cluster(0x0000)
+    ep.add_output_cluster(0x0001)
+    clus.update_attribute(0x0004, "Custom")
+    clus.update_attribute(0x0005, "Model")
+
+    ep = dev.add_endpoint(4)
+    ep.status = zigpy.endpoint.Status.ZDO_INIT
+    ep.profile_id = 260
+    ep.device_type = profiles.zha.DeviceType.PUMP
+    clus = ep.add_input_cluster(0x0006)
+    app.device_initialized(dev)
+
+    # Make an attribute unsupported
+    clus.add_unsupported_attribute(0x0000)
+
+    await app.shutdown()
+    del clus
+    del ep
+    del dev
+
+    def remove_cluster(device):
+        device.endpoints.pop(4)
+        return device
+
+    # Simulate a quirk that removes the entire endpoint
+    with patch("zigpy.quirks.get_device", side_effect=remove_cluster):
+        # The application should still load
+        app = await make_app_with_db(db)
+
+    dev = app.get_device(ieee)
+    assert 4 not in dev.endpoints
     await app.shutdown()
 
 
 async def test_last_seen(tmp_path):
     db = tmp_path / "test.db"
-    app = await make_app(db)
+    app = await make_app_with_db(db)
 
     ieee = make_ieee()
     app.handle_join(99, ieee, 0)
@@ -848,22 +897,22 @@ async def test_last_seen(tmp_path):
     ep.device_type = profiles.zha.DeviceType.PUMP
     clus = ep.add_input_cluster(0)
     ep.add_output_cluster(1)
-    clus._update_attribute(4, "Custom")
-    clus._update_attribute(5, "Model")
+    clus.update_attribute(4, "Custom")
+    clus.update_attribute(5, "Model")
     app.device_initialized(dev)
 
     old_last_seen = dev.last_seen
     await app.shutdown()
 
     # The `last_seen` of a joined device persists
-    app = await make_app(db)
+    app = await make_app_with_db(db)
     dev = app.get_device(ieee=ieee)
     await app.shutdown()
 
     next_last_seen = dev.last_seen
     assert abs(next_last_seen - old_last_seen) < 0.01
 
-    app = await make_app(db)
+    app = await make_app_with_db(db)
     dev = app.get_device(ieee=ieee)
 
     # Last-seen is only written to the db every 30s (no write case)
@@ -873,12 +922,12 @@ async def test_last_seen(tmp_path):
 
     await app.shutdown()
 
-    app = await make_app(db)
+    app = await make_app_with_db(db)
     dev = app.get_device(ieee=ieee)
     assert dev.last_seen == next_last_seen  # no change
     await app.shutdown()
 
-    app = await make_app(db)
+    app = await make_app_with_db(db)
     dev = app.get_device(ieee=ieee)
 
     # Last-seen is only written to the db every 30s (write case)
@@ -889,7 +938,7 @@ async def test_last_seen(tmp_path):
     await app.shutdown()
 
     # And it will be updated when the database next loads
-    app = await make_app(db)
+    app = await make_app_with_db(db)
     dev = app.get_device(ieee=ieee)
     assert dev.last_seen >= next_last_seen + 35  # updated
     await app.shutdown()
@@ -953,12 +1002,12 @@ async def test_appdb_network_backups(tmp_path, backup_factory):  # noqa: F811
 
     backup = backup_factory()
 
-    app1 = await make_app(db)
+    app1 = await make_app_with_db(db)
     app1.backups.add_backup(backup)
     await app1.shutdown()
 
     # The backup is reloaded from the database as well
-    app2 = await make_app(db)
+    app2 = await make_app_with_db(db)
     assert len(app2.backups.backups) == 1
     assert app2.backups.backups[0] == backup
 
@@ -969,7 +1018,7 @@ async def test_appdb_network_backups(tmp_path, backup_factory):  # noqa: F811
     await app2.shutdown()
 
     # The database will contain only the single backup
-    app3 = await make_app(db)
+    app3 = await make_app_with_db(db)
     assert len(app3.backups.backups) == 1
     assert app3.backups.backups[0] == new_backup
     assert app3.backups.backups[0] != backup
