@@ -50,6 +50,7 @@ async def test_new_exception(ota_mock):
             {
                 conf.CONF_DATABASE: "/dev/null",
                 conf.CONF_DEVICE: {conf.CONF_DEVICE_PATH: "/dev/null"},
+                conf.CONF_STARTUP_ENERGY_SCAN: False,
             }
         )
     assert db_mck.call_count == 1
@@ -67,6 +68,7 @@ async def test_new_exception(ota_mock):
                 {
                     conf.CONF_DATABASE: "/dev/null",
                     conf.CONF_DEVICE: {conf.CONF_DEVICE_PATH: "/dev/null"},
+                    conf.CONF_STARTUP_ENERGY_SCAN: False,
                 }
             )
     assert db_mck.call_count == 2
@@ -167,7 +169,7 @@ async def _remove(
 
     app.devices[ieee] = device
     await app.remove(ieee)
-    for i in range(1, 20):
+    for _i in range(1, 20):
         await asyncio.sleep(0)
     assert ieee not in app.devices
 
@@ -191,7 +193,7 @@ async def test_remove_with_failed_zdo(app, ieee):
 async def test_remove_nonexistent(app, ieee):
     with patch.object(app, "_remove_device", AsyncMock()) as remove_device:
         await app.remove(ieee)
-        for i in range(1, 20):
+        for _i in range(1, 20):
             await asyncio.sleep(0)
         assert ieee not in app.devices
         assert remove_device.await_count == 0
@@ -422,7 +424,7 @@ async def test_remove_parent_devices(app, make_initialized_device):
 
     with p1, p2, p3, p4, p5, p6, p7, p8:
         await app.remove(end_device.ieee)
-        for i in range(1, 60):
+        for _i in range(1, 60):
             await asyncio.sleep(0)
 
         assert end_device.zdo.leave.await_count == 1
@@ -447,6 +449,7 @@ async def test_startup_log_on_uninitialized_device(ieee, caplog):
         {
             conf.CONF_DATABASE: "/dev/null",
             conf.CONF_DEVICE: {conf.CONF_DEVICE_PATH: "/dev/null"},
+            conf.CONF_STARTUP_ENERGY_SCAN: False,
         }
     )
     assert "Device is partially initialized" in caplog.text
@@ -559,10 +562,46 @@ async def test_form_network(app):
         == t.KeyData(b"ZigBeeAlliance09")
     )
 
-    assert nwk_info1.channel == 15
+    assert nwk_info1.channel in (11, 15, 20, 25)
 
 
-async def test_startup_formed(app):
+@mock.patch("zigpy.util.pick_optimal_channel", mock.Mock(return_value=22))
+async def test_form_network_find_best_channel(app):
+    orig_start_network = app.start_network
+
+    async def start_network(*args, **kwargs):
+        start_network.await_count += 1
+
+        if start_network.await_count == 1:
+            raise NetworkNotFormed()
+
+        return await orig_start_network(*args, **kwargs)
+
+    start_network.await_count = 0
+    app.start_network = start_network
+
+    with patch.object(app, "write_network_info") as write:
+        with patch.object(
+            app.backups, "create_backup", wraps=app.backups.create_backup
+        ) as create_backup:
+            await app.form_network()
+
+    assert start_network.await_count == 2
+
+    # A temporary network will be formed first
+    nwk_info1 = write.mock_calls[0].kwargs["network_info"]
+    assert nwk_info1.channel == 11
+
+    # Then, after the scan, a better channel is chosen
+    nwk_info2 = write.mock_calls[1].kwargs["network_info"]
+    assert nwk_info2.channel == 22
+
+    # Only a single backup will be present
+    assert create_backup.await_count == 1
+
+
+async def test_startup_formed():
+    app = make_app({conf.CONF_STARTUP_ENERGY_SCAN: False})
     app.start_network = AsyncMock()
     app.form_network = AsyncMock()
     app.permit = AsyncMock()
@@ -574,7 +613,8 @@ async def test_startup_formed(app):
     assert app.permit.await_count == 1
 
 
-async def test_startup_not_formed(app):
+async def test_startup_not_formed():
+    app = make_app({conf.CONF_STARTUP_ENERGY_SCAN: False})
     app.start_network = AsyncMock()
     app.form_network = AsyncMock()
     app.load_network_info = AsyncMock(
@@ -600,7 +640,8 @@ async def test_startup_not_formed(app):
     assert app.backups.restore_backup.await_count == 0
 
 
-async def test_startup_not_formed_with_backup(app):
+async def test_startup_not_formed_with_backup():
+    app = make_app({conf.CONF_STARTUP_ENERGY_SCAN: False})
     app.start_network = AsyncMock()
     app.load_network_info = AsyncMock(side_effect=[NetworkNotFormed(), None])
     app.permit = AsyncMock()
@@ -614,46 +655,6 @@ async def test_startup_not_formed_with_backup(app):
     app.backups.restore_backup.assert_called_once_with(sentinel.NEW_BACKUP)
 
 
-async def test_deprecated_properties_and_methods(app):
-    with pytest.deprecated_call():
-        assert app.state.network_information is app.state.network_info
-
-    with pytest.deprecated_call():
-        assert app.state.node_information is app.state.node_info
-
-    app.shutdown = AsyncMock()
-    app.state = MagicMock()
-
-    with pytest.deprecated_call():
-        await app.pre_shutdown()
-
-    assert app.shutdown.await_count == 1
-
-    with pytest.deprecated_call():
-        assert app.nwk is app.state.node_info.nwk
-
-    with pytest.deprecated_call():
-        assert app.ieee is app.state.node_info.ieee
-
-    with pytest.deprecated_call():
-        assert app.pan_id is app.state.network_info.pan_id
-
-    with pytest.deprecated_call():
-        assert app.extended_pan_id is app.state.network_info.extended_pan_id
-
-    with pytest.deprecated_call():
-        assert app.network_key is app.state.network_info.network_key
-
-    with pytest.deprecated_call():
-        assert app.channel is app.state.network_info.channel
-
-    with pytest.deprecated_call():
-        assert app.channels is app.state.network_info.channel_mask
-
-    with pytest.deprecated_call():
-        assert app.nwk_update_id is app.state.network_info.nwk_update_id
-
-
 async def test_startup_backup():
     app = make_app({conf.CONF_NWK_BACKUP_ENABLED: True})
 
@@ -672,13 +673,24 @@ async def test_startup_no_backup():
     p.assert_not_called()
 
 
-async def test_startup_failure_transient_error():
+def with_attributes(obj, **attrs):
+    for k, v in attrs.items():
+        setattr(obj, k, v)
+
+    return obj
+
+
+@pytest.mark.parametrize(
+    "error",
+    [
+        with_attributes(OSError("Network is unreachable"), errno=errno.ENETUNREACH),
+        ConnectionRefusedError(),
+    ],
+)
+async def test_startup_failure_transient_error(error):
     app = make_app({conf.CONF_NWK_BACKUP_ENABLED: False})
 
-    err = OSError("Network is unreachable")
-    err.errno = errno.ENETUNREACH
-
-    with patch.object(app, "connect", side_effect=[err]):
+    with patch.object(app, "connect", side_effect=[error]):
         with pytest.raises(TransientConnectionError):
             await app.startup()
 
@@ -869,7 +881,6 @@ def test_build_source_route_no_relays(app):
 
 
 async def test_send_mrequest(app, packet):
-
     status, msg = await app.mrequest(
         group_id=0xABCD,
         profile=0x1234,
@@ -895,7 +906,6 @@ async def test_send_mrequest(app, packet):
 
 
 async def test_send_broadcast(app, packet):
-
     status, msg = await app.broadcast(
         profile=0x1234,
         cluster=0x0006,
@@ -949,11 +959,11 @@ async def test_packet_received_new_device_zdo_announce(app, device, zdo_packet):
 
     zdo_data = zigpy.zdo.ZDO(None)._serialize(
         zdo_t.ZDOCmd.Device_annce,
-        *dict(
-            NWKAddr=device.nwk,
-            IEEEAddr=device.ieee,
-            Capability=0x00,
-        ).values()
+        *{
+            "NWKAddr": device.nwk,
+            "IEEEAddr": device.ieee,
+            "Capability": 0x00,
+        }.values(),
     )
 
     zdo_packet.cluster_id = zdo_t.ZDOCmd.Device_annce
@@ -983,23 +993,23 @@ async def test_packet_received_new_device_discovery(app, device, zdo_packet):
             packet.cluster_id, packet.data.serialize()
         )
         assert args == list(
-            dict(
-                NWKAddrOfInterest=device.nwk,
-                RequestType=zdo_t.AddrRequestType.Single,
-                StartIndex=0,
-            ).values()
+            {
+                "NWKAddrOfInterest": device.nwk,
+                "RequestType": zdo_t.AddrRequestType.Single,
+                "StartIndex": 0,
+            }.values()
         )
 
         zdo_data = zigpy.zdo.ZDO(None)._serialize(
             zdo_t.ZDOCmd.IEEE_addr_rsp,
-            *dict(
-                Status=zdo_t.Status.SUCCESS,
-                IEEEAddr=device.ieee,
-                NWKAddr=device.nwk,
-                NumAssocDev=0,
-                StartIndex=0,
-                NWKAddrAssocDevList=[],
-            ).values()
+            *{
+                "Status": zdo_t.Status.SUCCESS,
+                "IEEEAddr": device.ieee,
+                "NWKAddr": device.nwk,
+                "NumAssocDev": 0,
+                "StartIndex": 0,
+                "NWKAddrAssocDevList": [],
+            }.values(),
         )
 
         # Receive the IEEE address reply
@@ -1018,7 +1028,7 @@ async def test_packet_received_new_device_discovery(app, device, zdo_packet):
     await asyncio.sleep(0.1)
 
     app.handle_join.assert_called_once_with(
-        nwk=device.nwk, ieee=device.ieee, parent_nwk=None
+        nwk=device.nwk, ieee=device.ieee, parent_nwk=None, handle_rejoin=False
     )
 
     zigpy_device = app.get_device(ieee=device.ieee)
@@ -1026,6 +1036,69 @@ async def test_packet_received_new_device_discovery(app, device, zdo_packet):
     assert zigpy_device.rssi == zdo_packet.rssi
 
 
+@patch("zigpy.device.Device.initialize", AsyncMock())
+async def test_packet_received_ieee_no_rejoin(app, device, zdo_packet, caplog):
+    device.is_initialized = True
+    app.devices[device.ieee] = device
+
+    app.handle_join = MagicMock(wraps=app.handle_join)
+
+    zdo_data = zigpy.zdo.ZDO(None)._serialize(
+        zdo_t.ZDOCmd.IEEE_addr_rsp,
+        *{
+            "Status": zdo_t.Status.SUCCESS,
+            "IEEEAddr": device.ieee,
+            "NWKAddr": device.nwk,
+        }.values(),
+    )
+
+    zdo_packet.cluster_id = zdo_t.ZDOCmd.IEEE_addr_rsp
+    zdo_packet.data = t.SerializableBytes(
+        t.uint8_t(zdo_packet.tsn).serialize() + zdo_data
+    )
+    app.packet_received(zdo_packet)
+
+    assert "joined the network" not in caplog.text
+
+    app.handle_join.assert_called_once_with(
+        nwk=device.nwk, ieee=device.ieee, parent_nwk=None, handle_rejoin=False
+    )
+
+    assert len(device.schedule_group_membership_scan.mock_calls) == 0
+    assert len(device.schedule_initialize.mock_calls) == 0
+
+
+@patch("zigpy.device.Device.initialize", AsyncMock())
+async def test_packet_received_ieee_rejoin(app, device, zdo_packet, caplog):
+    device.is_initialized = True
+    app.devices[device.ieee] = device
+
+    app.handle_join = MagicMock(wraps=app.handle_join)
+
+    zdo_data = zigpy.zdo.ZDO(None)._serialize(
+        zdo_t.ZDOCmd.IEEE_addr_rsp,
+        *{
+            "Status": zdo_t.Status.SUCCESS,
+            "IEEEAddr": device.ieee,
+            "NWKAddr": device.nwk + 1,  # NWK has changed
+        }.values(),
+    )
+
+    zdo_packet.cluster_id = zdo_t.ZDOCmd.IEEE_addr_rsp
+    zdo_packet.data = t.SerializableBytes(
+        t.uint8_t(zdo_packet.tsn).serialize() + zdo_data
+    )
+    app.packet_received(zdo_packet)
+
+    assert "joined the network" not in caplog.text
+
+    app.handle_join.assert_called_once_with(
+        nwk=device.nwk, ieee=device.ieee, parent_nwk=None, handle_rejoin=False
+    )
+
+    assert len(device.schedule_initialize.mock_calls) == 1
+
+
 async def test_bad_zdo_packet_received(app, device):
     device.is_initialized = True
     app.devices[device.ieee] = device
@@ -1086,12 +1159,12 @@ async def test_request_future_matching(app, make_initialized_device):
         disable_default_response=False,
         direction=foundation.Direction.Server_to_Client,
         args=(),
-        kwargs=dict(
-            field_control=0,
-            manufacturer_code=0x1234,
-            image_type=0x5678,
-            current_file_version=0x11112222,
-        ),
+        kwargs={
+            "field_control": 0,
+            "manufacturer_code": 0x1234,
+            "image_type": 0x5678,
+            "current_file_version": 0x11112222,
+        },
     )
 
     packet = t.ZigbeePacket(
@@ -1149,12 +1222,12 @@ async def test_request_callback_matching(app, make_initialized_device):
         disable_default_response=False,
         direction=foundation.Direction.Server_to_Client,
         args=(),
-        kwargs=dict(
-            field_control=0,
-            manufacturer_code=0x1234,
-            image_type=0x5678,
-            current_file_version=0x11112222,
-        ),
+        kwargs={
+            "field_control": 0,
+            "manufacturer_code": 0x1234,
+            "image_type": 0x5678,
+            "current_file_version": 0x11112222,
+        },
     )
 
     packet = t.ZigbeePacket(
@@ -1180,9 +1253,7 @@ async def test_request_callback_matching(app, make_initialized_device):
         assert app._req_listeners[device]
 
         asyncio.get_running_loop().call_soon(app.packet_received, packet)
-
         asyncio.get_running_loop().call_soon(app.packet_received, packet)
-
         asyncio.get_running_loop().call_soon(app.packet_received, packet)
 
         await asyncio.sleep(0.1)
@@ -1191,3 +1262,131 @@ async def test_request_callback_matching(app, make_initialized_device):
         assert mock_callback.mock_calls == [mock.call(req_hdr, req_cmd)] * 3
 
     assert not app._req_listeners[device]
+
+
+async def test_energy_scan_default(app):
+    await app.startup()
+
+    raw_scan_results = [
+        170,
+        191,
+        181,
+        165,
+        179,
+        169,
+        196,
+        163,
+        174,
+        162,
+        190,
+        186,
+        191,
+        178,
+        204,
+        187,
+    ]
+    coordinator = app._device
+    coordinator.zdo.Mgmt_NWK_Update_req = AsyncMock(
+        return_value=[
+            zdo_t.Status.SUCCESS,
+            t.Channels.ALL_CHANNELS,
+            29,
+            10,
+            raw_scan_results,
+        ]
+    )
+
+    results = await app.energy_scan(
+        channels=t.Channels.ALL_CHANNELS, duration_exp=2, count=1
+    )
+
+    assert len(results) == 16
+    assert results == dict(zip(range(11, 26 + 1), raw_scan_results))
+
+
+async def test_energy_scan_not_implemented(app):
+    """Energy scanning still "works" even when the radio doesn't implement it."""
+    await app.startup()
+    app._device.zdo.Mgmt_NWK_Update_req.side_effect = asyncio.TimeoutError()
+
+    results = await app.energy_scan(
+        channels=t.Channels.ALL_CHANNELS, duration_exp=2, count=1
+    )
+    assert results == {c: 0 for c in range(11, 26 + 1)}
+
+
+@pytest.mark.parametrize(
+    "scan, message_present",
+    [
+        ({c: 0 for c in t.Channels.ALL_CHANNELS}, False),
+        ({c: 255 for c in t.Channels.ALL_CHANNELS}, True),
+    ],
+)
+async def test_startup_energy_scan(app, caplog, scan, message_present):
+    with mock.patch.object(app, "energy_scan", return_value=scan):
+        with caplog.at_level(logging.WARNING):
+            await app.startup()
+
+    if message_present:
+        assert "Zigbee channel 15 utilization is 100.00%" in caplog.text
+    else:
+        assert "Zigbee channel" not in caplog.text
+
+
+async def test_startup_broadcast_failure_due_to_interference(app, caplog):
+    err = DeliveryError(
+        "Failed to deliver packet: <TXStatus.MAC_CHANNEL_ACCESS_FAILURE: 225>", 225
+    )
+
+    with mock.patch.object(app, "permit", side_effect=err):
+        with caplog.at_level(logging.WARNING):
+            await app.startup()
+
+    # The application will still start up, however
+    assert "Failed to send startup broadcast" in caplog.text
+    assert "interference" in caplog.text
+
+
+async def test_startup_broadcast_failure_other(app, caplog):
+    with mock.patch.object(app, "permit", side_effect=DeliveryError("Error", 123)):
+        with pytest.raises(DeliveryError, match="^Error$"):
+            await app.startup()
+
+
+@patch("zigpy.application.CHANNEL_CHANGE_SETTINGS_RELOAD_DELAY_S", 0.1)
+@patch("zigpy.application.CHANNEL_CHANGE_BROADCAST_DELAY_S", 0.01)
+async def test_move_network_to_new_channel(app):
+    async def nwk_update(*args, **kwargs):
+        async def inner():
+            await asyncio.sleep(
+                zigpy.application.CHANNEL_CHANGE_SETTINGS_RELOAD_DELAY_S * 5
+            )
+            NwkUpdate = args[0]
+            app.state.network_info.channel = list(NwkUpdate.ScanChannels)[0]
+            app.state.network_info.nwk_update_id = NwkUpdate.nwkUpdateId
+
+        asyncio.create_task(inner())  # noqa: RUF006
+
+    await app.startup()
+
+    assert app.state.network_info.channel != 26
+
+    with patch.object(
+        app._device.zdo, "Mgmt_NWK_Update_req", side_effect=nwk_update
+    ) as mock_update:
+        await app.move_network_to_channel(new_channel=26, num_broadcasts=10)
+
+    assert app.state.network_info.channel == 26
+    assert len(mock_update.mock_calls) == 1
+
+
+async def test_move_network_to_new_channel_noop(app):
+    await app.startup()
+
+    old_channel = app.state.network_info.channel
+
+    with patch("zigpy.zdo.broadcast") as mock_broadcast:
+        await app.move_network_to_channel(new_channel=old_channel)
+
+    assert app.state.network_info.channel == old_channel
+    assert len(mock_broadcast.mock_calls) == 0
