@@ -292,7 +292,6 @@ async def test_ikea_refresh_list_locked(mock_get, ikea_prov, ikea_image_with_ver
 
 @patch("aiohttp.ClientSession.get")
 async def test_ikea_refresh_list_failed(mock_get, ikea_prov):
-
     mock_get.return_value.__aenter__.return_value.json = AsyncMock(side_effect=[[]])
 
     mock_get.return_value.__aenter__.return_value.status = 434
@@ -353,7 +352,7 @@ def test_filestore_scan(file_image_name):
 def test_filestore_scan_exc(file_image_name):
     ota_file = file_image_name()
     with patch("builtins.open", mock.mock_open()) as mock_file:
-        mock_file.side_effect = IOError()
+        mock_file.side_effect = OSError()
 
         r = ota_p.FileImage.scan_image(ota_file)
         assert r is None
@@ -388,7 +387,7 @@ async def test_filestore_fetch_image(file_image):
 
 async def test_filestore_fetch_image_exc(file_image):
     with mock.patch("builtins.open", mock.mock_open()) as mock_file:
-        mock_file.side_effect = IOError()
+        mock_file.side_effect = OSError()
 
         r = await ota_p.FileImage.fetch_image(file_image)
         assert r is None
