@@ -24,17 +24,19 @@ def test_registry():
 
 
 def test_attributes():
-    for cluster_id, cluster in zcl.Cluster._registry.items():
+    for _cluster_id, cluster in zcl.Cluster._registry.items():
         for attrid, attr in cluster.attributes.items():
             assert 0 <= attrid <= 0xFFFF
             assert isinstance(attr, zcl.foundation.ZCLAttributeDef)
             assert attr.id == attrid
             assert attr.name
             assert attr.type
+            assert callable(attr.type.deserialize)
+            assert callable(attr.type.serialize)
 
 
 def _test_commands(cmdattr):
-    for cluster_id, cluster in zcl.Cluster._registry.items():
+    for _cluster_id, cluster in zcl.Cluster._registry.items():
         for cmdid, cmdspec in getattr(cluster, cmdattr).items():
             assert 0 <= cmdid <= 0xFF
 
@@ -42,6 +44,10 @@ def _test_commands(cmdattr):
             assert isinstance(cmdspec, zcl.foundation.ZCLCommandDef)
             assert issubclass(cmdspec.schema, types.Struct)
 
+            for field in cmdspec.schema.fields:
+                assert callable(field.type.deserialize)
+                assert callable(field.type.serialize)
+
 
 def test_server_commands():
     _test_commands("server_commands")
@@ -53,7 +59,7 @@ def test_client_commands():
 
 def test_ep_attributes():
     seen = set()
-    for cluster_id, cluster in zcl.Cluster._registry.items():
+    for _cluster_id, cluster in zcl.Cluster._registry.items():
         assert isinstance(cluster.ep_attribute, str)
         assert re.match(r"^[a-z_][a-z0-9_]*$", cluster.ep_attribute)
         assert cluster.ep_attribute not in seen
@@ -303,7 +309,7 @@ async def test_ota_handle_image_block_no_img(ota_cluster):
             .as_tuple(skip_missing=True)
         ),
         manufacturer=None,
-        tsn=ANY
+        tsn=ANY,
     )
     ota_cluster.reply.reset_mock()
 
@@ -318,7 +324,7 @@ async def test_ota_handle_image_block_no_img(ota_cluster):
             .as_tuple(skip_missing=True)
         ),
         manufacturer=None,
-        tsn=ANY
+        tsn=ANY,
     )
 
 
