@@ -135,7 +135,7 @@ async def test_retry_once():
 async def _test_retryable(exception, retry_exceptions, n, tries=3, delay=0.001):
     counter = 0
 
-    @util.retryable(retry_exceptions)
+    @util.retryable(retry_exceptions, tries=tries, delay=delay)
     async def count(x, y, z):
         assert x == y == z == 9
         nonlocal counter
@@ -145,7 +145,7 @@ async def _test_retryable(exception, retry_exceptions, n, tries=3, delay=0.001):
             exc._counter = counter
             raise exc
 
-    await count(9, 9, 9, tries=tries, delay=delay)
+    await count(9, 9, 9)
     return counter
 
 
@@ -557,3 +557,72 @@ async def test_dynamic_bounded_semaphore_errors(event_loop):
     assert not sem.locked()
     assert sem.value == 1
     assert sem.max_value == 1
+
+
+@pytest.mark.parametrize(
+    "plot",
+    [
+        r"""
+            11  #########
+            12  #################################################################
+            13  ##################################################################
+            14  ################################################################
+            15  ##############
+            16  ######################################################
+            17  #########################
+            18  ##########
+            19  ###########
+            20  ################
+            21  ###########
+            22  ###########
+            23  #
+            24  ##
+           [25] #################
+            26  ##
+        """,
+        r"""
+            11  ##################
+            12
+            13
+            14  ########
+            15  ##################
+            16  ########
+            17
+            18
+            19  ########
+            20  ##################
+            21  ########
+            22
+            23
+            24  ########
+           [25] ##################
+            26
+        """,
+        r"""
+            11  ##################
+            12  #
+            13  #
+            14  ########
+            15  ##################
+            16  ########
+            17
+            18
+            19  ########
+           [20] ##################
+            21  ########
+            22
+            23
+            24  ########
+            25  ##################
+            26  ########
+        """,
+    ],
+)
+def test_picking_optimal_channel(plot):
+    expected_channel = int(plot.split("[")[1].split("]")[0])
+    plot = plot.replace("[", " ").replace("]", " ")
+    channel_energy = {
+        int(line.split()[0]): line.count("#") for line in plot.strip().splitlines()
+    }
+
+    assert util.pick_optimal_channel(channel_energy) == expected_channel
