@@ -8,13 +8,20 @@ import errno
 import logging
 import os
 import random
+import sys
 import time
 import typing
-from typing import Any
+from typing import Any, Coroutine, TypeVar
+
+if sys.version_info[:2] < (3, 11):
+    from async_timeout import timeout as asyncio_timeout  # pragma: no cover
+else:
+    from asyncio import timeout as asyncio_timeout  # pragma: no cover
 
 import zigpy.appdb
 import zigpy.backups
 import zigpy.config as conf
+import zigpy.const as const
 import zigpy.device
 import zigpy.exceptions
 import zigpy.group
@@ -38,12 +45,18 @@ TRANSIENT_CONNECTION_ERRORS = {
     errno.ENETUNREACH,
 }
 
+ENERGY_SCAN_WARN_THRESHOLD = 0.75 * 255
+_R = TypeVar("_R")
+
+CHANNEL_CHANGE_BROADCAST_DELAY_S = 1.0
+CHANNEL_CHANGE_SETTINGS_RELOAD_DELAY_S = 1.0
+
 
 class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
     SCHEMA = conf.CONFIG_SCHEMA
     SCHEMA_DEVICE = conf.SCHEMA_DEVICE
 
-    def __init__(self, config: dict):
+    def __init__(self, config: dict) -> None:
         self.devices: dict[t.EUI64, zigpy.device.Device] = {}
         self.state: zigpy.state.State = zigpy.state.State()
         self._listeners = {}
@@ -53,6 +66,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         self._listeners = {}
         self._ota = zigpy.ota.OTA(self)
         self._send_sequence = 0
+        self._tasks: set[asyncio.Future[Any]] = set()
 
         self._concurrent_requests_semaphore = zigpy.util.DynamicBoundedSemaphore(
             self._config[conf.CONF_MAX_CONCURRENT_REQUESTS]
@@ -66,6 +80,18 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
             collections.deque[zigpy.listeners.BaseRequestListener],
         ] = collections.defaultdict(lambda: collections.deque([]))
 
+    def create_task(
+        self, target: Coroutine[Any, Any, _R], name: str | None = None
+    ) -> asyncio.Task[_R]:
+        """Create a task and store a reference to it until the task completes.
+
+        target: target to call.
+        """
+        task = asyncio.get_running_loop().create_task(target, name=name)
+        self._tasks.add(task)
+        task.add_done_callback(self._tasks.remove)
+        return task
+
     async def _load_db(self) -> None:
         """Restore save state."""
         database_file = self.config[conf.CONF_DATABASE]
@@ -79,9 +105,8 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         self.topology.add_listener(self._dblistener)
         await self._dblistener.load()
 
-    async def initialize(self, *, auto_form: bool = False):
-        """
-        Starts the network on a connected radio, optionally forming one with random
+    async def initialize(self, *, auto_form: bool = False) -> None:
+        """Starts the network on a connected radio, optionally forming one with random
         settings if necessary.
         """
 
@@ -122,7 +147,32 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         await self.start_network()
 
         # Some radios erroneously permit joins on startup
-        await self.permit(0)
+        try:
+            await self.permit(0)
+        except zigpy.exceptions.DeliveryError as e:
+            if e.status != t.MACStatus.MAC_CHANNEL_ACCESS_FAILURE:
+                raise
+
+            # Some radios (like the Conbee) can fail to deliver the startup broadcast
+            # due to interference
+            LOGGER.warning("Failed to send startup broadcast: %s", e)
+            LOGGER.warning(const.INTERFERENCE_MESSAGE)
+
+        if self.config[conf.CONF_STARTUP_ENERGY_SCAN]:
+            # Each scan period is 15.36ms. Scan for at least 200ms (2^4 + 1 periods) to
+            # pick up WiFi beacon frames.
+            results = await self.energy_scan(
+                channels=t.Channels.ALL_CHANNELS, duration_exp=4, count=1
+            )
+            LOGGER.debug("Startup energy scan: %s", results)
+
+            if results[self.state.network_info.channel] > ENERGY_SCAN_WARN_THRESHOLD:
+                LOGGER.warning(
+                    "Zigbee channel %s utilization is %0.2f%%!",
+                    self.state.network_info.channel,
+                    100 * results[self.state.network_info.channel] / 255,
+                )
+                LOGGER.warning(const.INTERFERENCE_MESSAGE)
 
         if self.config[conf.CONF_NWK_BACKUP_ENABLED]:
             self.backups.start_periodic_backups(
@@ -136,10 +186,8 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
                 period=(60 * self.config[zigpy.config.CONF_TOPO_SCAN_PERIOD])
             )
 
-    async def startup(self, *, auto_form: bool = False):
-        """
-        Starts a network, optionally forming one with random settings if necessary.
-        """
+    async def startup(self, *, auto_form: bool = False) -> None:
+        """Starts a network, optionally forming one with random settings if necessary."""
 
         try:
             await self.connect()
@@ -148,7 +196,9 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
             LOGGER.error("Couldn't start application", exc_info=e)
             await self.shutdown()
 
-            if isinstance(e, OSError) and e.errno in TRANSIENT_CONNECTION_ERRORS:
+            if isinstance(e, ConnectionError) or (
+                isinstance(e, OSError) and e.errno in TRANSIENT_CONNECTION_ERRORS
+            ):
                 raise zigpy.exceptions.TransientConnectionError() from e
 
             raise
@@ -174,10 +224,87 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
 
         return app
 
-    async def form_network(self):
-        """
-        Writes random network settings to the coordinator.
-        """
+    async def energy_scan(
+        self, channels: t.Channels, duration_exp: int, count: int
+    ) -> dict[int, float]:
+        """Runs an energy detection scan and returns the per-channel scan results."""
+        try:
+            rsp = await self._device.zdo.Mgmt_NWK_Update_req(
+                zigpy.zdo.types.NwkUpdate(
+                    ScanChannels=channels,
+                    ScanDuration=duration_exp,
+                    ScanCount=count,
+                )
+            )
+        except (asyncio.TimeoutError, zigpy.exceptions.DeliveryError):
+            LOGGER.warning("Coordinator does not support energy scanning")
+            scanned_channels = channels
+            energy_values = [0] * scanned_channels
+        else:
+            _, scanned_channels, _, _, energy_values = rsp
+
+        return dict(zip(scanned_channels, energy_values))
+
+    async def _move_network_to_channel(
+        self, new_channel: int, new_nwk_update_id: int
+    ) -> None:
+        """Broadcasts the channel migration update request."""
+        # Default implementation for radios that migrate via a loopback ZDO request
+        await self._device.zdo.Mgmt_NWK_Update_req(
+            zigpy.zdo.types.NwkUpdate(
+                ScanChannels=zigpy.types.Channels.from_channel_list([new_channel]),
+                ScanDuration=zigpy.zdo.types.NwkUpdate.CHANNEL_CHANGE_REQ,
+                nwkUpdateId=new_nwk_update_id,
+            )
+        )
+
+    async def move_network_to_channel(
+        self, new_channel: int, *, num_broadcasts: int = 5
+    ) -> None:
+        """Moves the network to a new channel."""
+        if self.state.network_info.channel == new_channel:
+            return
+
+        new_nwk_update_id = (self.state.network_info.nwk_update_id + 1) % 0xFF
+
+        for attempt in range(num_broadcasts):
+            LOGGER.info(
+                "Broadcasting migration to channel %s (%s of %s)",
+                new_channel,
+                attempt + 1,
+                num_broadcasts,
+            )
+
+            await zigpy.zdo.broadcast(
+                app=self,
+                command=zigpy.zdo.types.ZDOCmd.Mgmt_NWK_Update_req,
+                grpid=None,
+                radius=30,  # Explicitly set the maximum radius
+                broadcast_address=zigpy.types.BroadcastAddress.ALL_DEVICES,
+                NwkUpdate=zigpy.zdo.types.NwkUpdate(
+                    ScanChannels=zigpy.types.Channels.from_channel_list([new_channel]),
+                    ScanDuration=zigpy.zdo.types.NwkUpdate.CHANNEL_CHANGE_REQ,
+                    nwkUpdateId=new_nwk_update_id,
+                ),
+            )
+
+            await asyncio.sleep(CHANNEL_CHANGE_BROADCAST_DELAY_S)
+
+        # Move the coordinator itself, if supported
+        await self._move_network_to_channel(
+            new_channel=new_channel, new_nwk_update_id=new_nwk_update_id
+        )
+
+        # Wait for settings to update
+        while self.state.network_info.channel != new_channel:
+            LOGGER.info("Waiting for channel change to take effect")
+            await self.load_network_info(load_devices=False)
+            await asyncio.sleep(CHANNEL_CHANGE_SETTINGS_RELOAD_DELAY_S)
+
+        LOGGER.info("Successfully migrated to channel %d", new_channel)
+
+    async def form_network(self, *, fast: bool = False) -> None:
+        """Writes random network settings to the coordinator."""
 
         # First, make the settings consistent and randomly generate missing values
         channel = self.config[conf.CONF_NWK][conf.CONF_NWK_CHANNEL]
@@ -186,10 +313,31 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         extended_pan_id = self.config[conf.CONF_NWK][conf.CONF_NWK_EXTENDED_PAN_ID]
         network_key = self.config[conf.CONF_NWK][conf.CONF_NWK_KEY]
         tc_address = self.config[conf.CONF_NWK][conf.CONF_NWK_TC_ADDRESS]
+        stack_specific = {}
+
+        if fast:
+            # Indicate to the radio library that the network is ephemeral
+            stack_specific["form_quickly"] = True
 
         if pan_id is None:
             pan_id = random.SystemRandom().randint(0x0001, 0xFFFE + 1)
 
+        if channel is None and fast:
+            # Don't run an energy scan if this is an ephemeral network
+            channel = next(iter(channels))
+        elif channel is None and not fast:
+            # We can't run an energy scan without a running network on most radios
+            try:
+                await self.start_network()
+            except zigpy.exceptions.NetworkNotFormed:
+                await self.form_network(fast=True)
+                await self.start_network()
+
+            channel_energy = await self.energy_scan(
+                channels=t.Channels.ALL_CHANNELS, duration_exp=4, count=1
+            )
+            channel = zigpy.util.pick_optimal_channel(channel_energy, channels=channels)
+
         if extended_pan_id is None:
             # TODO: exclude `FF:FF:FF:FF:FF:FF:FF:FF` and possibly more reserved EPIDs
             extended_pan_id = t.ExtendedPanId(os.urandom(8))
@@ -200,17 +348,13 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         if tc_address is None:
             tc_address = t.EUI64.UNKNOWN
 
-        # Override `channels` with a single channel if one is explicitly set
-        if channel is not None:
-            channels = t.Channels.from_channel_list([channel])
-
         network_info = zigpy.state.NetworkInfo(
             extended_pan_id=extended_pan_id,
             pan_id=pan_id,
             nwk_update_id=self.config[conf.CONF_NWK][conf.CONF_NWK_UPDATE_ID],
             nwk_manager_id=0x0000,
             channel=channel,
-            channel_mask=channels,
+            channel_mask=t.Channels.from_channel_list([channel]),
             security_level=5,
             network_key=zigpy.state.Key(
                 key=network_key,
@@ -228,7 +372,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
             children=[],
             key_table=[],
             nwk_addresses={},
-            stack_specific={},
+            stack_specific=stack_specific,
         )
 
         node_info = zigpy.state.NodeInfo(
@@ -246,6 +390,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
             ),
             counter_increment=0,
             allow_incomplete=True,
+            create_new=(not fast),
         )
 
     async def shutdown(self) -> None:
@@ -258,10 +403,8 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
 
         await self.disconnect()
 
-    def add_device(self, ieee: t.EUI64, nwk: t.NWK):
-        """
-        Creates a zigpy `Device` object with the provided IEEE and NWK addresses.
-        """
+    def add_device(self, ieee: t.EUI64, nwk: t.NWK) -> zigpy.device.Device:
+        """Creates a zigpy `Device` object with the provided IEEE and NWK addresses."""
 
         assert isinstance(ieee, t.EUI64)
         # TODO: Shut down existing device
@@ -270,7 +413,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         self.devices[ieee] = dev
         return dev
 
-    def device_initialized(self, device):
+    def device_initialized(self, device: zigpy.device.Device) -> None:
         """Used by a device to signal that it is initialized"""
         LOGGER.debug("Device is initialized %s", device)
 
@@ -297,8 +440,9 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         dev.cancel_initialization()
 
         LOGGER.info("Removing device 0x%04x (%s)", dev.nwk, ieee)
-        asyncio.create_task(
-            self._remove_device(dev, remove_children=remove_children, rejoin=rejoin)
+        self.create_task(
+            self._remove_device(dev, remove_children=remove_children, rejoin=rejoin),
+            f"remove_device-nwk={dev.nwk!r}-ieee={ieee!r}",
         )
         if dev.node_desc is not None and dev.node_desc.is_end_device:
             parents = []
@@ -334,12 +478,12 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
     ) -> None:
         """Send a remove request then pop the device."""
         try:
-            await asyncio.wait_for(
-                device.zdo.leave(remove_children=remove_children, rejoin=rejoin),
-                timeout=30
+            async with asyncio_timeout(
+                30
                 if device.node_desc is not None and device.node_desc.is_end_device
-                else 7,
-            )
+                else 7
+            ):
+                await device.zdo.leave(remove_children=remove_children, rejoin=rejoin)
         except (zigpy.exceptions.DeliveryError, asyncio.TimeoutError) as ex:
             LOGGER.debug("Sending 'zdo_leave_req' failed: %s", ex)
 
@@ -366,8 +510,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         dst_addressing: None
         | (t.Addressing.Group | t.Addressing.IEEE | t.Addressing.NWK) = None,
     ) -> None:
-        """
-        Called when the radio library receives a packet.
+        """Called when the radio library receives a packet.
 
         Deprecated, will be removed.
         """
@@ -432,21 +575,29 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         if not sender.initializing and not sender.is_initialized:
             sender.schedule_initialize()
 
-    def handle_join(self, nwk: t.NWK, ieee: t.EUI64, parent_nwk: t.NWK) -> None:
-        """
-        Called when a device joins or announces itself on the network.
-        """
+    def handle_join(
+        self,
+        nwk: t.NWK,
+        ieee: t.EUI64,
+        parent_nwk: t.NWK,
+        *,
+        handle_rejoin: bool = True,
+    ) -> None:
+        """Called when a device joins or announces itself on the network."""
 
         ieee = t.EUI64(ieee)
 
         try:
             dev = self.get_device(ieee=ieee)
-            LOGGER.info("Device 0x%04x (%s) joined the network", nwk, ieee)
-            new_join = False
         except KeyError:
             dev = self.add_device(ieee, nwk)
             LOGGER.info("New device 0x%04x (%s) joined the network", nwk, ieee)
             new_join = True
+        else:
+            if handle_rejoin:
+                LOGGER.info("Device 0x%04x (%s) joined the network", nwk, ieee)
+
+            new_join = False
 
         if dev.nwk != nwk:
             LOGGER.debug("Device %s changed id (0x%04x => 0x%04x)", ieee, dev.nwk, nwk)
@@ -463,14 +614,12 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         elif not dev.is_initialized:
             # Re-initialize partially-initialized devices but don't emit "device_joined"
             dev.schedule_initialize()
-        else:
+        elif handle_rejoin:
             # Rescan groups for devices that are not newly joining and initialized
             dev.schedule_group_membership_scan()
 
     def handle_leave(self, nwk: t.NWK, ieee: t.EUI64):
-        """
-        Called when a device has left the network.
-        """
+        """Called when a device has left the network."""
         LOGGER.info("Device 0x%04x (%s) left the network", nwk, ieee)
 
         try:
@@ -481,22 +630,22 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
             self.listener_event("device_left", dev)
 
     def handle_relays(self, nwk: t.NWK, relays: list[t.NWK]) -> None:
-        """
-        Called when a list of relaying devices is received.
-        """
+        """Called when a list of relaying devices is received."""
         try:
             device = self.get_device(nwk=nwk)
         except KeyError:
             LOGGER.warning("Received relays from an unknown device: %s", nwk)
-            asyncio.create_task(self._discover_unknown_device(nwk))
+            self.create_task(
+                self._discover_unknown_device(nwk),
+                f"discover_unknown_device_from_relays-nwk={nwk!r}",
+            )
         else:
             # `relays` is a property with a setter that emits an event
             device.relays = relays
 
     @classmethod
     async def probe(cls, device_config: dict[str, Any]) -> bool | dict[str, Any]:
-        """
-        Probes the device specified by `device_config` and returns valid device settings
+        """Probes the device specified by `device_config` and returns valid device settings
         if the radio supports the device. If the device is not supported, `False` is
         returned.
         """
@@ -516,45 +665,37 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
 
     @abc.abstractmethod
     async def connect(self):
-        """
-        Connect to the radio hardware and verify that it is compatible with the library.
+        """Connect to the radio hardware and verify that it is compatible with the library.
         This method should be stateless if the connection attempt fails.
         """
         raise NotImplementedError()  # pragma: no cover
 
     @abc.abstractmethod
     async def disconnect(self):
-        """
-        Disconnects from the radio hardware and shuts down the network.
-        """
+        """Disconnects from the radio hardware and shuts down the network."""
         raise NotImplementedError()  # pragma: no cover
 
     @abc.abstractmethod
     async def start_network(self):
-        """
-        Starts a Zigbee network with settings currently stored in the radio hardware.
-        """
+        """Starts a Zigbee network with settings currently stored in the radio hardware."""
         raise NotImplementedError()  # pragma: no cover
 
     @abc.abstractmethod
     async def force_remove(self, dev: zigpy.device.Device):
-        """
-        Instructs the radio to remove a device with a lower-level leave command. Not all
+        """Instructs the radio to remove a device with a lower-level leave command. Not all
         radios implement this.
         """
         raise NotImplementedError()  # pragma: no cover
 
     @abc.abstractmethod
     async def add_endpoint(self, descriptor: zdo_types.SimpleDescriptor):
-        """
-        Registers a new endpoint on the controlled device. Not all radios will implement
+        """Registers a new endpoint on the controlled device. Not all radios will implement
         this.
         """
         raise NotImplementedError()  # pragma: no cover
 
-    async def register_endpoints(self):
-        """
-        Registers all necessary endpoints.
+    async def register_endpoints(self) -> None:
+        """Registers all necessary endpoints.
         The exact order in which this method is called depends on the radio module.
         """
 
@@ -596,9 +737,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
 
     @contextlib.asynccontextmanager
     async def _limit_concurrency(self):
-        """
-        Async context manager to limit global coordinator request concurrency.
-        """
+        """Async context manager to limit global coordinator request concurrency."""
 
         start_time = time.monotonic()
         was_locked = self._concurrent_requests_semaphore.locked()
@@ -621,16 +760,12 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
 
     @abc.abstractmethod
     async def send_packet(self, packet: t.ZigbeePacket) -> None:
-        """
-        Send a Zigbee packet using the appropriate addressing mode and provided options.
-        """
+        """Send a Zigbee packet using the appropriate addressing mode and provided options."""
 
         raise NotImplementedError()  # pragma: no cover
 
     def build_source_route_to(self, dest: zigpy.device.Device) -> list[t.NWK] | None:
-        """
-        Compute a source route to the destination device.
-        """
+        """Compute a source route to the destination device."""
 
         if dest.relays is None:
             return None
@@ -638,7 +773,6 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         # TODO: utilize topology scanner information
         return dest.relays[::-1]
 
-    @zigpy.util.retryable_request
     async def request(
         self,
         device: zigpy.device.Device,
@@ -652,7 +786,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         expect_reply: bool = True,
         use_ieee: bool = False,
         extended_timeout: bool = False,
-    ):
+    ) -> tuple[zigpy.zcl.foundation.Status, str]:
         """Submit and send data out as an unicast transmission.
         :param device: destination device
         :param profile: Zigbee Profile ID to use for outgoing message
@@ -761,7 +895,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         sequence: t.uint8_t,
         data: bytes,
         broadcast_address: t.BroadcastAddress = t.BroadcastAddress.RX_ON_WHEN_IDLE,
-    ):
+    ) -> tuple[zigpy.zcl.foundation.Status, str]:
         """Submit and send data out as an unicast transmission.
         :param profile: Zigbee Profile ID to use for outgoing message
         :param cluster: cluster id where the message is being sent
@@ -797,9 +931,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         return (zigpy.zcl.foundation.Status.SUCCESS, "")
 
     async def _discover_unknown_device(self, nwk: t.NWK) -> None:
-        """
-        Discover the IEEE address of a device with an unknown NWK.
-        """
+        """Discover the IEEE address of a device with an unknown NWK."""
 
         return await zigpy.zdo.broadcast(
             app=self,
@@ -812,9 +944,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         )
 
     def _maybe_parse_zdo(self, packet: t.ZigbeePacket) -> None:
-        """
-        Attempt to parse an incoming packet as ZDO, to extract useful notifications.
-        """
+        """Attempt to parse an incoming packet as ZDO, to extract useful notifications."""
 
         # The current zigpy device may not exist if we receive a packet early
         try:
@@ -842,12 +972,12 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
 
             if status == zdo_types.Status.SUCCESS:
                 LOGGER.debug("Discovered IEEE address for NWK=%s: %s", nwk, ieee)
-                self.handle_join(nwk=nwk, ieee=ieee, parent_nwk=None)
+                self.handle_join(
+                    nwk=nwk, ieee=ieee, parent_nwk=None, handle_rejoin=False
+                )
 
     def packet_received(self, packet: t.ZigbeePacket) -> None:
-        """
-        Notify zigpy of a received Zigbee packet.
-        """
+        """Notify zigpy of a received Zigbee packet."""
 
         LOGGER.debug("Received a packet: %r", packet)
         assert packet.src is not None
@@ -864,7 +994,10 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
 
             if packet.src.addr_mode == t.AddrMode.NWK:
                 # Manually send a ZDO IEEE address request to discover the device
-                asyncio.create_task(self._discover_unknown_device(packet.src.address))
+                self.create_task(
+                    self._discover_unknown_device(packet.src.address),
+                    f"discover_unknown_device_from_packet-nwk={packet.src.address!r}",
+                )
 
             return
 
@@ -883,9 +1016,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
     def get_device_with_address(
         self, address: t.AddrModeAddress
     ) -> zigpy.device.Device:
-        """
-        Gets a `Device` object using the provided address mode address.
-        """
+        """Gets a `Device` object using the provided address mode address."""
 
         if address.addr_mode == t.AddrMode.NWK:
             return self.get_device(nwk=address.address)
@@ -907,9 +1038,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
             typing.Any,
         ],
     ) -> typing.Any:
-        """
-        Context manager to create a callback that is passed Zigbee responses.
-        """
+        """Context manager to create a callback that is passed Zigbee responses."""
 
         listener = zigpy.listeners.CallbackListener(
             device=src,
@@ -930,9 +1059,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         src: zigpy.device.Device,
         filters: list[zigpy.listeners.MatcherType],
     ) -> typing.Any:
-        """
-        Context manager to wait for a Zigbee response.
-        """
+        """Context manager to wait for a Zigbee response."""
 
         listener = zigpy.listeners.FutureListener(
             device=src,
@@ -948,18 +1075,15 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
             self._req_listeners[src].remove(listener)
 
     @abc.abstractmethod
-    async def permit_ncp(self, time_s: int = 60):
-        """
-        Permit joining on NCP.
+    async def permit_ncp(self, time_s: int = 60) -> None:
+        """Permit joining on NCP.
         Not all radios will require this method.
         """
         raise NotImplementedError()  # pragma: no cover
 
     @abc.abstractmethod
     async def permit_with_key(self, node: t.EUI64, code: bytes, time_s: int = 60):
-        """
-        Permit a node to join with the provided install code bytes.
-        """
+        """Permit a node to join with the provided install code bytes."""
         raise NotImplementedError()  # pragma: no cover
 
     @abc.abstractmethod
@@ -969,16 +1093,14 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         network_info: zigpy.state.NetworkInfo,
         node_info: zigpy.state.NodeInfo,
     ) -> None:
-        """
-        Writes network and node state to the radio hardware.
+        """Writes network and node state to the radio hardware.
         Any information not supported by the radio should be logged as a warning.
         """
         raise NotImplementedError()  # pragma: no cover
 
     @abc.abstractmethod
     async def load_network_info(self, *, load_devices: bool = False) -> None:
-        """
-        Loads network and node information from the radio hardware.
+        """Loads network and node information from the radio hardware.
 
         :param load_devices: if `False`, supplementary network information that may take
                              a while to load should be skipped. For example, device NWK
@@ -988,13 +1110,11 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
 
     @abc.abstractmethod
     async def reset_network_info(self) -> None:
-        """
-        Leaves the current network.
-        """
+        """Leaves the current network."""
 
         raise NotImplementedError()  # pragma: no cover
 
-    async def permit(self, time_s: int = 60, node: t.EUI64 | str | None = None):
+    async def permit(self, time_s: int = 60, node: t.EUI64 | str | None = None) -> None:
         """Permit joining on a specific node or all router nodes."""
         assert 0 <= time_s <= 254
         if node is not None:
@@ -1022,7 +1142,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
             0,
             broadcast_address=t.BroadcastAddress.ALL_ROUTERS_AND_COORDINATOR,
         )
-        return await self.permit_ncp(time_s)
+        await self.permit_ncp(time_s)
 
     def get_sequence(self) -> t.uint8_t:
         self._send_sequence = (self._send_sequence + 1) % 256
@@ -1031,8 +1151,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
     def get_device(
         self, ieee: t.EUI64 = None, nwk: t.NWK | int = None
     ) -> zigpy.device.Device:
-        """
-        Looks up a device in the `devices` dictionary based either on its NWK or IEEE
+        """Looks up a device in the `devices` dictionary based either on its NWK or IEEE
         address.
         """
 
@@ -1056,7 +1175,7 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         """Returns coordinator endpoint id for specified cluster id."""
         return DEFAULT_ENDPOINT_ID
 
-    def get_dst_address(self, cluster) -> zdo_types.MultiAddress:
+    def get_dst_address(self, cluster: zigpy.zcl.Cluster) -> zdo_types.MultiAddress:
         """Helper to get a dst address for bind/unbind operations.
 
         Allows radios to provide correct information especially for radios which listen
@@ -1085,58 +1204,14 @@ class ControllerApplication(zigpy.util.ListenableMixin, abc.ABC):
         self._config = self.SCHEMA(new_config)
 
     @property
-    def groups(self):
+    def groups(self) -> zigpy.group.Groups:
         return self._groups
 
     @property
-    def ota(self):
+    def ota(self) -> zigpy.ota.OTA:
         return self._ota
 
     @property
     def _device(self):
         """The device being controlled."""
         return self.get_device(ieee=self.state.node_info.ieee)
-
-    @zigpy.util.deprecated("await `app.shutdown()`")
-    async def pre_shutdown(self):
-        await self.shutdown()
-
-    @property
-    @zigpy.util.deprecated("use `app.state.node_info.nwk`")
-    def nwk(self):
-        return self.state.node_info.nwk
-
-    @property
-    @zigpy.util.deprecated("use `app.state.node_info.ieee`")
-    def ieee(self):
-        return self.state.node_info.ieee
-
-    @property
-    @zigpy.util.deprecated("use `app.state.network_info.pan_id`")
-    def pan_id(self):
-        return self.state.network_info.pan_id
-
-    @property
-    @zigpy.util.deprecated("use `app.state.network_info.extended_pan_id`")
-    def extended_pan_id(self):
-        return self.state.network_info.extended_pan_id
-
-    @property
-    @zigpy.util.deprecated("use `app.state.network_info.network_key`")
-    def network_key(self):
-        return self.state.network_info.network_key
-
-    @property
-    @zigpy.util.deprecated("use `app.state.network_info.channel`")
-    def channel(self):
-        return self.state.network_info.channel
-
-    @property
-    @zigpy.util.deprecated("use `app.state.network_info.channel_mask`")
-    def channels(self):
-        return self.state.network_info.channel_mask
-
-    @property
-    @zigpy.util.deprecated("use `app.state.network_info.nwk_update_id`")
-    def nwk_update_id(self):
-        return self.state.network_info.nwk_update_id
