@@ -1,10 +1,13 @@
+from __future__ import annotations
+
 import zigpy.types as t
 
+CONF_STARTUP_ENERGY_SCAN_DEFAULT = True
 CONF_MAX_CONCURRENT_REQUESTS_DEFAULT = 8
 CONF_NWK_BACKUP_ENABLED_DEFAULT = True
 CONF_NWK_BACKUP_PERIOD_DEFAULT = 24 * 60  # 24 hours
-CONF_NWK_CHANNEL_DEFAULT = 15
-CONF_NWK_CHANNELS_DEFAULT = [15, 20, 25]
+CONF_NWK_CHANNEL_DEFAULT = None
+CONF_NWK_CHANNELS_DEFAULT = [11, 15, 20, 25]
 CONF_NWK_EXTENDED_PAN_ID_DEFAULT = None
 CONF_NWK_PAN_ID_DEFAULT = None
 CONF_NWK_KEY_DEFAULT = None
