@@ -1,7 +1,8 @@
 """OTA support for Zigbee devices."""
+from __future__ import annotations
+
 import datetime
 import logging
-from typing import Optional
 
 import attr
 
@@ -11,6 +12,9 @@ from zigpy.config import (
     CONF_OTA_IKEA,
     CONF_OTA_INOVELLI,
     CONF_OTA_LEDVANCE,
+    CONF_OTA_PROVIDER_MANUF_IDS,
+    CONF_OTA_PROVIDER_URL,
+    CONF_OTA_REMOTE_PROVIDERS,
     CONF_OTA_SALUS,
     CONF_OTA_SONOFF,
     CONF_OTA_THIRDREALITY,
@@ -38,7 +42,7 @@ class CachedImage:
     cached_data = attr.ib(default=None)
 
     @classmethod
-    def new(cls, img: BaseOTAImage) -> "CachedImage":
+    def new(cls, img: BaseOTAImage) -> CachedImage:
         expiration = datetime.datetime.now() + cls.DEFAULT_EXPIRATION
         return cls(img, expiration)
 
@@ -60,7 +64,13 @@ class CachedImage:
     def version(self) -> int:
         return self.image.header.file_version
 
-    def should_update(self, manufacturer_id, img_type, ver, hw_ver=None) -> bool:
+    def should_update(
+        self,
+        manufacturer_id: t.uint16_t,
+        img_type: t.uint16_t,
+        ver: t.uint32_t,
+        hw_ver: None = None,
+    ) -> bool:
         """Check if it should upgrade"""
 
         if self.key != ImageKey(manufacturer_id, img_type):
@@ -82,7 +92,7 @@ class CachedImage:
 
         return True
 
-    def get_image_block(self, offset: t.uint32_t, size: t.uint8_t) -> bytes:
+    def get_image_block(self, offset: t.t.uint32_t, size: t.uint8_t) -> bytes:
         if (
             self.expires_on is not None
             and self.expires_on - datetime.datetime.now() < DELAY_EXPIRATION
@@ -101,7 +111,7 @@ class CachedImage:
 class OTA(zigpy.util.ListenableMixin):
     """OTA Manager."""
 
-    def __init__(self, app: ControllerApplicationType, *args, **kwargs):
+    def __init__(self, app: ControllerApplicationType, *args, **kwargs) -> None:
         self._app: ControllerApplicationType = app
         self._image_cache: dict[ImageKey, CachedImage] = {}
         self._not_initialized = True
@@ -122,13 +132,24 @@ class OTA(zigpy.util.ListenableMixin):
         if ota_config[CONF_OTA_THIRDREALITY]:
             self.add_listener(zigpy.ota.provider.ThirdReality())
 
+        for provider_config in ota_config[CONF_OTA_REMOTE_PROVIDERS]:
+            self.add_listener(
+                zigpy.ota.provider.RemoteProvider(
+                    url=provider_config[CONF_OTA_PROVIDER_URL],
+                    manufacturer_ids=provider_config[CONF_OTA_PROVIDER_MANUF_IDS],
+                )
+            )
+
     async def initialize(self) -> None:
         await self.async_event("initialize_provider", self._app.config[CONF_OTA])
         self._not_initialized = False
 
     async def get_ota_image(
-        self, manufacturer_id, image_type, model=None
-    ) -> Optional[CachedImage]:
+        self,
+        manufacturer_id: t.uint16_t,
+        image_type: t.uint16_t,
+        model: str | None = None,
+    ) -> CachedImage | None:
         if manufacturer_id in (
             zigpy.ota.provider.Salus.MANUFACTURER_ID,
         ):  # Salus/computime do not pass a useful image_type
