@@ -5,6 +5,7 @@ from abc import ABC, abstractmethod
 import asyncio
 from collections import defaultdict
 import datetime
+import hashlib
 import io
 import logging
 import os
@@ -34,11 +35,11 @@ class Basic(zigpy.util.LocalLogMixin, ABC):
 
     REFRESH = datetime.timedelta(hours=12)
 
-    def __init__(self):
-        self.config = {}
-        self._cache = {}
+    def __init__(self) -> None:
+        self.config: dict[str, str | int] = {}
+        self._cache: dict[ImageKey, BaseOTAImage] = {}
         self._is_enabled = False
-        self._locks = defaultdict(asyncio.Semaphore)
+        self._locks: defaultdict[asyncio.Semaphore] = defaultdict(asyncio.Semaphore)
         self._last_refresh = None
 
     @abstractmethod
@@ -77,7 +78,7 @@ class Basic(zigpy.util.LocalLogMixin, ABC):
     def enable(self) -> None:
         self._is_enabled = True
 
-    def update_expiration(self):
+    def update_expiration(self) -> None:
         self._last_refresh = datetime.datetime.now()
 
     @property
@@ -92,7 +93,7 @@ class Basic(zigpy.util.LocalLogMixin, ABC):
 
         return datetime.datetime.now() - self._last_refresh > self.REFRESH
 
-    def log(self, lvl, msg, *args, **kwargs):
+    def log(self, lvl: int, msg: str, *args, **kwargs) -> None:
         """Log a message"""
         msg = f"{self.__class__.__name__}: {msg}"
         return LOGGER.log(lvl, msg, *args, **kwargs)
@@ -107,7 +108,7 @@ class IKEAImage:
     url = attr.ib(default=None)
 
     @classmethod
-    def new(cls, data):
+    def new(cls, data: dict[str, str | int]) -> IKEAImage:
         res = cls(data["fw_manufacturer_id"], data["fw_image_type"])
         res.file_version = data["fw_file_version_MSB"] << 16
         res.file_version |= data["fw_file_version_LSB"]
@@ -116,7 +117,7 @@ class IKEAImage:
         return res
 
     @property
-    def key(self):
+    def key(self) -> ImageKey:
         return ImageKey(self.manufacturer_id, self.image_type)
 
     async def fetch_image(self) -> BaseOTAImage | None:
@@ -549,7 +550,7 @@ class FileImage:
 
 
 class FileStore(Basic):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
         self._ota_dir = None
 
@@ -581,7 +582,7 @@ class FileStore(Basic):
 
         self._cache.clear()
         loop = asyncio.get_event_loop()
-        for root, dirs, files in os.walk(self._ota_dir):
+        for root, _dirs, files in os.walk(self._ota_dir):
             for file in files:
                 if file in SKIP_OTA_FILES:
                     continue
@@ -627,7 +628,7 @@ class INOVELLIImage:
     url = attr.ib()
 
     @classmethod
-    def from_json(cls, obj):
+    def from_json(cls, obj: dict[str, str | int]) -> INOVELLIImage:
         version = int(obj["version"], 16)
 
         # Old Inovelli OTA JSON versions were in hex, they then switched back to decimal
@@ -642,7 +643,7 @@ class INOVELLIImage:
         )
 
     @property
-    def key(self):
+    def key(self) -> ImageKey:
         return ImageKey(self.manufacturer_id, self.image_type)
 
     async def fetch_image(self) -> BaseOTAImage | None:
@@ -666,7 +667,7 @@ class INOVELLIImage:
 class Inovelli(Basic):
     """Inovelli OTA Firmware provider."""
 
-    UPDATE_URL = "https://files.inovelli.com/firmware/firmware.json"
+    UPDATE_URL = "https://files.inovelli.com/firmware/firmware-zha.json"
     MANUFACTURER_ID = 4655
     HEADERS = {"accept": "application/json"}
 
@@ -695,7 +696,7 @@ class Inovelli(Basic):
         self.debug("Finished downloading firmware update list")
         self._cache.clear()
 
-        for model, firmwares in fw_lst.items():
+        for _model, firmwares in fw_lst.items():
             for firmware in firmwares:
                 img = INOVELLIImage.from_json(firmware)
 
@@ -796,3 +797,107 @@ class ThirdReality(Basic):
 
     async def filter_get_image(self, key: ImageKey) -> bool:
         return key.manufacturer_id not in self.MANUFACTURER_IDS
+
+
+@attr.s
+class RemoteImage:
+    binary_url = attr.ib()
+    file_version = attr.ib()
+    image_type = attr.ib()
+    manufacturer_id = attr.ib()
+    changelog = attr.ib()
+    checksum = attr.ib()
+
+    # Optional
+    min_hardware_version = attr.ib()
+    max_hardware_version = attr.ib()
+    min_current_file_version = attr.ib()
+    max_current_file_version = attr.ib()
+
+    @classmethod
+    def from_json(cls, obj: dict[str, typing.Any]) -> RemoteImage:
+        return cls(
+            binary_url=obj["binary_url"],
+            file_version=obj["file_version"],
+            image_type=obj["image_type"],
+            manufacturer_id=obj["manufacturer_id"],
+            changelog=obj["changelog"],
+            checksum=obj["checksum"],
+            min_hardware_version=obj.get("min_hardware_version"),
+            max_hardware_version=obj.get("max_hardware_version"),
+            min_current_file_version=obj.get("min_current_file_version"),
+            max_current_file_version=obj.get("max_current_file_version"),
+        )
+
+    @property
+    def key(self) -> ImageKey:
+        return ImageKey(self.manufacturer_id, self.image_type)
+
+    async def fetch_image(self) -> BaseOTAImage:
+        async with aiohttp.ClientSession() as req:
+            LOGGER.debug("Downloading %s for %s", self.binary_url, self.key)
+            async with req.get(self.binary_url) as rsp:
+                data = await rsp.read()
+
+        algorithm, checksum = self.checksum.split(":")
+        hasher = hashlib.new(algorithm)
+        await asyncio.get_running_loop().run_in_executor(None, hasher.update, data)
+
+        if hasher.hexdigest() != checksum:
+            raise ValueError(
+                f"Image checksum is invalid: expected {self.checksum},"
+                f" got {hasher.hexdigest()}"
+            )
+
+        ota_image, _ = parse_ota_image(data)
+
+        LOGGER.debug("Finished downloading %s", self)
+        return ota_image
+
+
+class RemoteProvider(Basic):
+    """Generic zigpy OTA URL provider."""
+
+    HEADERS = {"accept": "application/json"}
+
+    def __init__(self, url: str, manufacturer_ids: list[int] | None) -> None:
+        super().__init__()
+
+        self.url = url
+        self.manufacturer_ids = manufacturer_ids
+
+    async def initialize_provider(self, ota_config: dict) -> None:
+        self.info("OTA provider enabled")
+        await self.refresh_firmware_list()
+        self.enable()
+
+    async def refresh_firmware_list(self) -> None:
+        if self._locks[LOCK_REFRESH].locked():
+            return
+
+        async with self._locks[LOCK_REFRESH]:
+            async with aiohttp.ClientSession(headers=self.HEADERS) as req:
+                async with req.get(self.url) as rsp:
+                    if not (200 <= rsp.status <= 299):
+                        self.warning(
+                            "Couldn't download '%s': %s/%s",
+                            rsp.url,
+                            rsp.status,
+                            rsp.reason,
+                        )
+                        return
+                    fw_lst = await rsp.json()
+
+        self.debug("Finished downloading firmware update list")
+        self._cache.clear()
+        for obj in fw_lst:
+            img = RemoteImage.from_json(obj)
+            self._cache[img.key] = img
+
+        self.update_expiration()
+
+    async def filter_get_image(self, key: ImageKey) -> bool:
+        if not self.manufacturer_ids:
+            return False
+
+        return key.manufacturer_id not in self.manufacturer_ids
