@@ -6,8 +6,8 @@ import collections
 import functools
 import inspect
 import logging
-import sys
 import traceback
+import types
 import typing
 import warnings
 
@@ -23,24 +23,24 @@ LOGGER = logging.getLogger(__name__)
 
 
 class ListenableMixin:
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
         self._listeners: dict[int, tuple[typing.Callable, bool]] = {}
 
-    def _add_listener(self, listener, include_context):
+    def _add_listener(self, listener: typing.Any, include_context: bool) -> int:
         id_ = id(listener)
         while id_ in self._listeners:
             id_ += 1
         self._listeners[id_] = (listener, include_context)
         return id_
 
-    def add_listener(self, listener):
+    def add_listener(self, listener: typing.Any) -> int:
         return self._add_listener(listener, include_context=False)
 
-    def add_context_listener(self, listener):
+    def add_context_listener(self, listener: CatchingTaskMixin) -> int:
         return self._add_listener(listener, include_context=True)
 
-    def listener_event(self, method_name, *args):
+    def listener_event(self, method_name: str, *args) -> list[typing.Any | None]:
         result = []
         for listener, include_context in self._listeners.values():
             method = getattr(listener, method_name, None)
@@ -62,7 +62,7 @@ class ListenableMixin:
                 )
         return result
 
-    async def async_event(self, method_name, *args):
+    async def async_event(self, method_name: str, *args) -> list[typing.Any]:
         tasks = []
         for listener, include_context in self._listeners.values():
             method = getattr(listener, method_name, None)
@@ -97,30 +97,31 @@ class LocalLogMixin:
     def log(self, lvl: int, msg: str, *args, **kwargs):  # pragma: no cover
         pass
 
-    def _log(self, lvl: int, msg: str, *args, **kwargs):
-        if sys.version_info >= (3, 8):
-            # We have to exclude log, _log, and info
-            return self.log(lvl, msg, *args, stacklevel=4, **kwargs)
-
-        return self.log(lvl, msg, *args, **kwargs)
+    def _log(self, lvl: int, msg: str, *args, **kwargs) -> None:
+        return self.log(lvl, msg, *args, stacklevel=4, **kwargs)
 
     def exception(self, msg, *args, **kwargs):
         return self._log(logging.ERROR, msg, *args, **kwargs)
 
-    def debug(self, msg, *args, **kwargs):
+    def debug(self, msg: str, *args, **kwargs) -> None:
         return self._log(logging.DEBUG, msg, *args, **kwargs)
 
-    def info(self, msg, *args, **kwargs):
+    def info(self, msg: str, *args, **kwargs) -> None:
         return self._log(logging.INFO, msg, *args, **kwargs)
 
-    def warning(self, msg, *args, **kwargs):
+    def warning(self, msg: str, *args, **kwargs) -> None:
         return self._log(logging.WARNING, msg, *args, **kwargs)
 
     def error(self, msg, *args, **kwargs):
         return self._log(logging.ERROR, msg, *args, **kwargs)
 
 
-async def retry(func, retry_exceptions, tries=3, delay=0.1):
+async def retry(
+    func: typing.Callable[[], typing.Awaitable[typing.Any]],
+    retry_exceptions: typing.Iterable[BaseException],
+    tries: int = 3,
+    delay: int | float = 0.1,
+) -> typing.Any:
     """Retry a function in case of exception
 
     Only exceptions in `retry_exceptions` will be retried.
@@ -128,8 +129,7 @@ async def retry(func, retry_exceptions, tries=3, delay=0.1):
     while True:
         LOGGER.debug("Tries remaining: %s", tries)
         try:
-            r = await func()
-            return r
+            return await func()
         except retry_exceptions:
             if tries <= 1:
                 raise
@@ -137,18 +137,18 @@ async def retry(func, retry_exceptions, tries=3, delay=0.1):
             await asyncio.sleep(delay)
 
 
-def retryable(retry_exceptions, tries=1, delay=0.1):
-    """Return a decorator which makes a function able to be retried
-
-    This adds "tries" and "delay" keyword arguments to the function. Only
-    exceptions in `retry_exceptions` will be retried.
+def retryable(
+    retry_exceptions: typing.Iterable[BaseException], tries: int = 1, delay: float = 0.1
+) -> typing.Callable:
+    """Return a decorator which makes a function able to be retried.
+    Only exceptions in `retry_exceptions` will be retried.
     """
 
-    def decorator(func):
+    def decorator(func: typing.Callable) -> typing.Callable:
         nonlocal tries, delay
 
         @functools.wraps(func)
-        def wrapper(*args, tries=tries, delay=delay, **kwargs):
+        def wrapper(*args, **kwargs):
             if tries <= 1:
                 return func(*args, **kwargs)
             return retry(
@@ -163,7 +163,9 @@ def retryable(retry_exceptions, tries=1, delay=0.1):
     return decorator
 
 
-retryable_request = retryable((ZigbeeException, asyncio.TimeoutError))
+retryable_request = functools.partial(
+    retryable, (ZigbeeException, asyncio.TimeoutError)
+)
 
 
 def aes_mmo_hash_update(length: int, result: bytes, data: bytes) -> tuple[int, bytes]:
@@ -265,12 +267,12 @@ class Request:
         """Send Future."""
         return self._sequence
 
-    def __enter__(self):
+    def __enter__(self) -> Request:
         """Return context manager."""
         self._pending[self.sequence] = self
         return self
 
-    def __exit__(self, exc_type, exc_value, exc_traceback):
+    def __exit__(self, exc_type: None, exc_value: None, exc_traceback: None) -> bool:
         """Clean up pending on exit."""
         if not self.result.done():
             self.result.cancel()
@@ -292,13 +294,20 @@ class Requests(dict):
 class CatchingTaskMixin(LocalLogMixin):
     """Allow creating tasks suppressing exceptions."""
 
+    _tasks: set[asyncio.Future[typing.Any]] = set()
+
     def create_catching_task(
         self,
         target: typing.Coroutine,
         exceptions: type[Exception] | tuple | None = None,
+        name: str | None = None,
     ) -> None:
         """Create a task."""
-        asyncio.create_task(self.catching_coro(target, exceptions))
+        task = asyncio.get_running_loop().create_task(
+            self.catching_coro(target, exceptions), name=name
+        )
+        self._tasks.add(task)
+        task.add_done_callback(self._tasks.remove)
 
     async def catching_coro(
         self,
@@ -323,9 +332,7 @@ class CatchingTaskMixin(LocalLogMixin):
 
 
 class DynamicBoundedSemaphore(asyncio.Semaphore):
-    """
-    `asyncio.BoundedSemaphore` with public interface to access and change the max value.
-    """
+    """`asyncio.BoundedSemaphore` with public interface to access and change the max value."""
 
     def __init__(self, value: int = 0) -> None:
         self._value: int = value
@@ -358,9 +365,7 @@ class DynamicBoundedSemaphore(asyncio.Semaphore):
 
     @max_value.setter
     def max_value(self, new_value: int) -> None:
-        """
-        Update the semaphore's max value.
-        """
+        """Update the semaphore's max value."""
         if new_value < 0:
             raise ValueError(f"Semaphore value must be >= 0: {new_value!r}")
 
@@ -380,9 +385,8 @@ class DynamicBoundedSemaphore(asyncio.Semaphore):
         """Returns True if semaphore cannot be acquired immediately."""
         return self._value <= 0
 
-    async def acquire(self):
-        """
-        Acquire a semaphore.
+    async def acquire(self) -> typing.Literal[True]:
+        """Acquire a semaphore.
 
         If the internal counter is larger than zero on entry, decrement it by one and
         return True immediately.  If it is zero on entry, block, waiting until some
@@ -409,8 +413,7 @@ class DynamicBoundedSemaphore(asyncio.Semaphore):
         return True
 
     def release(self) -> None:
-        """
-        Release a semaphore, incrementing the internal counter by one.
+        """Release a semaphore, incrementing the internal counter by one.
 
         When it was zero on entry and another coroutine is waiting for it to become
         larger than zero again, wake up that coroutine.
@@ -425,7 +428,12 @@ class DynamicBoundedSemaphore(asyncio.Semaphore):
         await self.acquire()
         return None
 
-    async def __aexit__(self, exc_type, exc, tb):
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc: BaseException | None,
+        traceback: types.TracebackType | None,
+    ) -> None:
         self.release()
 
     def __repr__(self) -> str:
@@ -438,9 +446,7 @@ class DynamicBoundedSemaphore(asyncio.Semaphore):
 
 
 def deprecated(message: str) -> typing.Callable[[typing.Callable], typing.Callable]:
-    """
-    Decorator that emits a DeprecationWarning when the function or property is accessed.
-    """
+    """Decorator that emits a DeprecationWarning when the function or property is accessed."""
 
     def decorator(function: typing.Callable) -> typing.Callable:
         @functools.wraps(function)
@@ -459,9 +465,7 @@ def deprecated(message: str) -> typing.Callable[[typing.Callable], typing.Callab
 def deprecated_attrs(
     mapping: dict[str, typing.Any]
 ) -> typing.Callable[[str], typing.Any]:
-    """
-    Create a module-level `__getattr__` function that remaps deprecated objects.
-    """
+    """Create a module-level `__getattr__` function that remaps deprecated objects."""
 
     def __getattr__(name: str) -> typing.Any:
         if name not in mapping:
@@ -479,3 +483,70 @@ def deprecated_attrs(
         return replacement
 
     return __getattr__
+
+
+def pick_optimal_channel(
+    channel_energy: dict[int, float],
+    channels: t.Channels = t.Channels.from_channel_list([11, 15, 20, 25]),
+    *,
+    kernel: list[float] = (0.1, 0.5, 1.0, 0.5, 0.1),
+    channel_penalty: dict[int, float] = {
+        11: 2.0,  # ZLL but WiFi interferes
+        12: 3.0,
+        13: 3.0,
+        14: 3.0,
+        15: 1.0,  # ZLL
+        16: 3.0,
+        17: 3.0,
+        18: 3.0,
+        19: 3.0,
+        20: 1.0,  # ZLL
+        21: 3.0,
+        22: 3.0,
+        23: 3.0,
+        24: 3.0,
+        25: 1.0,  # ZLL
+        26: 2.0,  # Not ZLL but WiFi can interfere in some regions
+    },
+) -> int:
+    """Scans all channels and picks the best one from the given mask."""
+    assert len(kernel) % 2 == 1
+    kernel_width = (len(kernel) - 1) // 2
+
+    # Scan all channels even if we're restricted to picking among a few, since
+    # nearby channels will affect our decision
+    assert set(channel_energy.keys()) == set(t.Channels.ALL_CHANNELS)  # type: ignore
+
+    # We don't know energies above channel 26 or below 11. Assume the scan results
+    # just continue indefinitely with the last-seen value.
+    ext_energies = (
+        [channel_energy[11]] * kernel_width
+        + [channel_energy[c] for c in t.Channels.ALL_CHANNELS]
+        + [channel_energy[26]] * kernel_width
+    )
+
+    # Incorporate the energies of nearby channels into our calculation by performing
+    # a discrete convolution with the provided kernel.
+    convolution = ext_energies[:]
+
+    for i in range(len(ext_energies)):
+        for j in range(-kernel_width, kernel_width + 1):
+            if 0 <= i + j < len(convolution):
+                convolution[i + j] += ext_energies[i] * kernel[kernel_width + j]
+
+    # Crop off the extended bounds, leaving us with an array of the original size
+    convolution = convolution[kernel_width:-kernel_width]
+
+    # Incorporate a penalty to avoid specific channels unless absolutely necessary.
+    # Adding `1` ensures the score is positive and the channel penalty gets applied even
+    # when the reported LQI is zero.
+    scores = {
+        c: (1 + convolution[c - 11]) * channel_penalty.get(c, 1.0)
+        for c in t.Channels.ALL_CHANNELS
+    }
+    optimal_channel = min(channels, key=lambda c: scores[c])
+
+    LOGGER.info("Optimal channel is %s", optimal_channel)
+    LOGGER.debug("Channel scores: %s", scores)
+
+    return optimal_channel
