@@ -1,21 +1,31 @@
 from __future__ import annotations
 
+import collections
+from datetime import datetime, timezone
 import enum
 import functools
+import itertools
 import logging
-from typing import Any, Sequence
+import types
+from typing import TYPE_CHECKING, Any, Iterable, Sequence
 import warnings
 
 from zigpy import util
 import zigpy.types as t
 from zigpy.typing import AddressingMode, EndpointType
 from zigpy.zcl import foundation
+from zigpy.zcl.foundation import BaseAttributeDefs, BaseCommandDefs
+
+if TYPE_CHECKING:
+    from zigpy.appdb import PersistingListener
+    from zigpy.endpoint import Endpoint
+
 
 LOGGER = logging.getLogger(__name__)
 
 
 def convert_list_schema(
-    schema: Sequence[type], command_id: int, is_reply: bool
+    schema: Sequence[type], command_id: int, direction: foundation.Direction
 ) -> type[t.Struct]:
     schema_dict = {}
 
@@ -30,7 +40,7 @@ def convert_list_schema(
 
     temp = foundation.ZCLCommandDef(
         schema=schema_dict,
-        direction=foundation.Direction._from_is_reply(is_reply),
+        direction=direction,
         id=command_id,
         name="schema",
     )
@@ -46,6 +56,15 @@ class ClusterType(enum.IntEnum):
 class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
     """A cluster on an endpoint"""
 
+    class AttributeDefs(BaseAttributeDefs):
+        pass
+
+    class ServerCommandDefs(BaseCommandDefs):
+        pass
+
+    class ClientCommandDefs(BaseCommandDefs):
+        pass
+
     # Custom clusters for quirks subclass Cluster but should not be stored in any global
     # registries, since they're device-specific and collide with existing clusters.
     _skip_registry: bool = False
@@ -60,74 +79,38 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
     # to remove the need to create 1024 "ManufacturerSpecificCluster" instances.
     cluster_id_range: tuple[t.uint16_t, t.uint16_t] = None
 
-    # Clusters contain attributes and both client and server commands
+    # Deprecated: clusters contain attributes and both client and server commands
     attributes: dict[int, foundation.ZCLAttributeDef] = {}
     client_commands: dict[int, foundation.ZCLCommandDef] = {}
     server_commands: dict[int, foundation.ZCLCommandDef] = {}
+    attributes_by_name: dict[str, foundation.ZCLAttributeDef] = {}
+    commands_by_name: dict[str, foundation.ZCLCommandDef] = {}
 
     # Internal caches and indices
     _registry: dict = {}
     _registry_range: dict = {}
 
-    _server_commands_idx: dict[str, int] = {}
-    _client_commands_idx: dict[str, int] = {}
-
-    attributes_by_name: dict[str, foundation.ZCLAttributeDef] = {}
-    commands_by_name: dict[str, foundation.ZCLCommandDef] = {}
-
-    def __init_subclass__(cls):
-        # Fail on deprecated attribute presence
-        for a in ("attributes", "client_commands", "server_commands"):
-            if not hasattr(cls, f"manufacturer_{a}"):
-                continue
-
-            raise TypeError(
-                f"`manufacturer_{a}` is deprecated. Copy the parent class's `{a}`"
-                f" dictionary and update it with your manufacturer-specific `{a}`. Make"
-                f" sure to specify that it is manufacturer-specific through the "
-                f" appropriate constructor or tuple!"
-            )
-
+    def __init_subclass__(cls) -> None:
         if cls.cluster_id is not None:
             cls.cluster_id = t.ClusterId(cls.cluster_id)
 
-        # Clear the caches and lookup tables. Their contents should correspond exactly
-        # to what's in their respective command/attribute dictionaries.
-        cls.attributes_by_name = {}
-        cls.commands_by_name = {}
-        cls._server_commands_idx = {}
-        cls._client_commands_idx = {}
-
-        # Compile command definitions
-        for commands, index in [
-            (cls.server_commands, cls._server_commands_idx),
-            (cls.client_commands, cls._client_commands_idx),
-        ]:
+        # Compile the old command definitions
+        for commands in [cls.server_commands, cls.client_commands]:
             for command_id, command in list(commands.items()):
                 if isinstance(command, tuple):
                     # Backwards compatibility with old command tuples
-                    name, schema, is_reply = command
+                    name, schema, direction = command
                     command = foundation.ZCLCommandDef(
                         id=command_id,
                         name=name,
-                        schema=convert_list_schema(schema, command_id, is_reply),
-                        is_reply=is_reply,
-                    )
-                else:
-                    command = command.replace(id=command_id)
-
-                if command.name in cls.commands_by_name:
-                    raise TypeError(
-                        f"Command name {command} is not unique in {cls}: {cls.commands_by_name}"
+                        schema=convert_list_schema(schema, command_id, direction),
+                        direction=direction,
                     )
 
-                index[command.name] = command.id
-
-                command = command.with_compiled_schema()
+                command = command.replace(id=command_id).with_compiled_schema()
                 commands[command.id] = command
-                cls.commands_by_name[command.name] = command
 
-        # Compile attributes
+        # Compile the old attribute definitions
         for attr_id, attr in list(cls.attributes.items()):
             if isinstance(attr, tuple):
                 if len(attr) == 2:
@@ -142,11 +125,77 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
                     type=attr_type,
                     is_manufacturer_specific=attr_manuf_specific,
                 )
-            else:
-                attr = attr.replace(id=attr_id)
 
-            cls.attributes[attr.id] = attr
-            cls.attributes_by_name[attr.name] = attr
+            cls.attributes[attr.id] = attr.replace(id=attr_id)
+
+        # Create new definitions from the old-style definitions
+        if cls.attributes and "AttributeDefs" not in cls.__dict__:
+            cls.AttributeDefs = types.new_class(
+                name="AttributeDefs",
+                bases=(BaseAttributeDefs,),
+            )
+
+            for attr in cls.attributes.values():
+                setattr(cls.AttributeDefs, attr.name, attr)
+
+        if cls.server_commands and "ServerCommandDefs" not in cls.__dict__:
+            cls.ServerCommandDefs = types.new_class(
+                name="ServerCommandDefs",
+                bases=(BaseCommandDefs,),
+            )
+
+            for command in cls.server_commands.values():
+                setattr(cls.ServerCommandDefs, command.name, command)
+
+        if cls.client_commands and "ClientCommandDefs" not in cls.__dict__:
+            cls.ClientCommandDefs = types.new_class(
+                name="ClientCommandDefs",
+                bases=(BaseCommandDefs,),
+            )
+
+            for command in cls.client_commands.values():
+                setattr(cls.ClientCommandDefs, command.name, command)
+
+        # Check the old definitions for duplicates
+        for old_defs in [cls.attributes, cls.server_commands, cls.client_commands]:
+            counts = collections.Counter(d.name for d in old_defs.values())
+
+            if len(counts) != sum(counts.values()):
+                duplicates = [n for n, c in counts.items() if c > 1]
+                raise TypeError(f"Duplicate definitions exist for {duplicates}")
+
+        # Populate the `name` attribute of every definition
+        for defs in (cls.ServerCommandDefs, cls.ClientCommandDefs, cls.AttributeDefs):
+            for name in dir(defs):
+                definition = getattr(defs, name)
+
+                if (
+                    isinstance(
+                        definition,
+                        (foundation.ZCLCommandDef, foundation.ZCLAttributeDef),
+                    )
+                    and definition.name is None
+                ):
+                    object.__setattr__(definition, "name", name)
+
+        # Compile the schemas
+        for defs in (cls.ServerCommandDefs, cls.ClientCommandDefs):
+            for name in dir(defs):
+                definition = getattr(defs, name)
+
+                if isinstance(definition, foundation.ZCLCommandDef):
+                    setattr(defs, definition.name, definition.with_compiled_schema())
+
+        # Recreate the old structures using the new-style definitions
+        cls.attributes = {attr.id: attr for attr in cls.AttributeDefs}
+        cls.client_commands = {cmd.id: cmd for cmd in cls.ClientCommandDefs}
+        cls.server_commands = {cmd.id: cmd for cmd in cls.ServerCommandDefs}
+        cls.attributes_by_name = {attr.name: attr for attr in cls.AttributeDefs}
+
+        all_cmds: Iterable[foundation.ZCLCommandDef] = itertools.chain(
+            cls.ClientCommandDefs, cls.ServerCommandDefs
+        )
+        cls.commands_by_name = {cmd.name: cmd for cmd in all_cmds}
 
         if cls._skip_registry:
             return
@@ -157,9 +206,10 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
         if cls.cluster_id_range is not None:
             cls._registry_range[cls.cluster_id_range] = cls
 
-    def __init__(self, endpoint: EndpointType, is_server: bool = True):
+    def __init__(self, endpoint: EndpointType, is_server: bool = True) -> None:
         self._endpoint: EndpointType = endpoint
         self._attr_cache: dict[int, Any] = {}
+        self._attr_last_updated: dict[int, datetime] = {}
         self.unsupported_attributes: set[int | str] = set()
         self._listeners = {}
         self._type: ClusterType = (
@@ -259,7 +309,9 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
         # Convert out-of-band dict schemas to struct schemas
         if isinstance(schema, (tuple, list)):
             schema = convert_list_schema(
-                command_id=command_id, schema=schema, is_reply=False
+                command_id=command_id,
+                schema=schema,
+                direction=foundation.Direction.Server_to_Client,
             )
 
         request = schema(*args, **kwargs)  # type:ignore[operator]
@@ -289,7 +341,6 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
 
         return hdr, request
 
-    @util.retryable_request
     async def request(
         self,
         general: bool,
@@ -338,7 +389,7 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
         manufacturer: int | t.uint16_t | None = None,
         tsn: int | t.uint8_t | None = None,
         **kwargs,
-    ):
+    ) -> None:
         hdr, request = self._create_request(
             general=general,
             command_id=command_id,
@@ -365,7 +416,7 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
         args: list[Any],
         *,
         dst_addressing: AddressingMode | None = None,
-    ):
+    ) -> None:
         self.debug(
             "Received command 0x%02X (TSN %d): %s", hdr.command_id, hdr.tsn, args
         )
@@ -438,7 +489,7 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
         allow_cache: bool = False,
         only_cache: bool = False,
         manufacturer: int | t.uint16_t | None = None,
-    ):
+    ) -> Any:
         success, failure = {}, {}
         attribute_ids: list[int] = []
         orig_attributes: dict[int, int | str] = {}
@@ -685,7 +736,7 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
             len(records) == 1 and records[0].status == foundation.Status.SUCCESS
         ):
             # we get a single success when all are supported
-            for attr in attributes.keys():
+            for attr in attributes:
                 self.remove_unsupported_attribute(attr)
         return res
 
@@ -695,7 +746,6 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
         *args,
         manufacturer: int | t.uint16_t | None = None,
         expect_reply: bool = True,
-        tries: int = 1,
         tsn: int | t.uint8_t | None = None,
         **kwargs,
     ):
@@ -708,7 +758,6 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
             *args,
             manufacturer=manufacturer,
             expect_reply=expect_reply,
-            tries=tries,
             tsn=tsn,
             **kwargs,
         )
@@ -744,26 +793,30 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
         return self._type == ClusterType.Server
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self.__class__.__name__
 
     @property
-    def endpoint(self):
+    def endpoint(self) -> Endpoint:
         return self._endpoint
 
     @property
     def commands(self):
-        return list(self._server_commands_idx.keys())
+        return list(self.ServerCommandDefs)
 
-    def update_attribute(self, attrid, value):
+    def update_attribute(self, attrid: int | t.uint16_t, value: Any) -> None:
         """Update specified attribute with specified value"""
         self._update_attribute(attrid, value)
 
-    def _update_attribute(self, attrid, value):
-        self._attr_cache[attrid] = value
-        self.listener_event("attribute_updated", attrid, value)
+    def _update_attribute(self, attrid: int | t.uint16_t, value: Any) -> None:
+        now = datetime.now(timezone.utc)
 
-    def log(self, lvl, msg, *args, **kwargs):
+        self._attr_cache[attrid] = value
+        self._attr_last_updated[attrid] = now
+
+        self.listener_event("attribute_updated", attrid, value, now)
+
+    def log(self, lvl: int, msg: str, *args, **kwargs) -> None:
         msg = "[%s:%s:0x%04x] " + msg
         args = (
             self._endpoint.device.name,
@@ -772,23 +825,26 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
         ) + args
         return LOGGER.log(lvl, msg, *args, **kwargs)
 
-    def __getattr__(self, name):
-        if name in self._client_commands_idx:
-            return functools.partial(
-                self.client_command, self._client_commands_idx[name]
-            )
-        elif name in self._server_commands_idx:
-            return functools.partial(self.command, self._server_commands_idx[name])
+    def __getattr__(self, name: str) -> functools.partial:
+        try:
+            cmd = getattr(self.ClientCommandDefs, name)
+        except AttributeError:
+            pass
         else:
-            raise AttributeError(f"No such command name: {name}")
+            return functools.partial(self.client_command, cmd.id)
+
+        try:
+            cmd = getattr(self.ServerCommandDefs, name)
+        except AttributeError:
+            pass
+        else:
+            return functools.partial(self.command, cmd.id)
+
+        raise AttributeError(f"No such command name: {name}")
 
     def get(self, key: int | str, default: Any | None = None) -> Any:
         """Get cached attribute."""
-        try:
-            attr_def = self.find_attribute(key)
-        except KeyError:
-            return default
-
+        attr_def = self.find_attribute(key)
         return self._attr_cache.get(attr_def.id, default)
 
     def __getitem__(self, key: int | str) -> Any:
@@ -807,7 +863,6 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
         *args,
         manufacturer: int | t.uint16_t | None = None,
         expect_reply: bool = True,
-        tries: int = 1,
         tsn: int | t.uint8_t | None = None,
         **kwargs,
     ):
@@ -832,7 +887,6 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
             *args,
             manufacturer=manufacturer,
             expect_reply=expect_reply,
-            tries=tries,
             tsn=tsn,
             **kwargs,
         )
@@ -928,17 +982,19 @@ class Cluster(util.ListenableMixin, util.CatchingTaskMixin):
 
 
 class ClusterPersistingListener:
-    def __init__(self, applistener, cluster):
+    def __init__(self, applistener: PersistingListener, cluster: Cluster) -> None:
         self._applistener = applistener
         self._cluster = cluster
 
-    def attribute_updated(self, attrid, value):
-        self._applistener.attribute_updated(self._cluster, attrid, value)
+    def attribute_updated(
+        self, attrid: int | t.uint16_t, value: Any, timestamp: datetime
+    ) -> None:
+        self._applistener.attribute_updated(self._cluster, attrid, value, timestamp)
 
-    def cluster_command(self, *args, **kwargs):
+    def cluster_command(self, *args, **kwargs) -> None:
         pass
 
-    def general_command(self, *args, **kwargs):
+    def general_command(self, *args, **kwargs) -> None:
         pass
 
     def unsupported_attribute_added(self, attrid: int) -> None:
